@@ -31,6 +31,10 @@
 #include "utils.h"
 #include "runner.h"
 
+#ifndef __GNUC__
+#define __attribute__(x)
+#endif
+
 static gboolean
 parse_first_part(run_request *r, DBusMessage *msg, DBusMessageIter *iter)
 {
@@ -48,7 +52,11 @@ parse_first_part(run_request *r, DBusMessage *msg, DBusMessageIter *iter)
 		goto malformed;
 	dbus_message_iter_recurse(iter, &sub_iter);
 	/* Add default path for the programs we start */
+#if defined(__FreeBSD__)
+	tmpstr = g_strdup_printf("PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/sbin:/usr/X11R6/sbin:/bin:/usr/bin:/usr/local/bin:/usr/X11R6/bin:%s", getenv("PATH"));
+#else
 	tmpstr = g_strdup_printf("PATH=/sbin:/usr/sbin:/bin:/usr/bin:%s", getenv("PATH"));
+#endif
 	r->environment = get_string_array(&sub_iter, tmpstr);
 
 	/* Then argv */
@@ -111,7 +119,7 @@ handle_start(DBusConnection *con, DBusMessage *msg)
 	DBusMessage *reply;
 	DBusMessageIter iter;
 	run_request *r;
-	GPid pid;
+	GPid pid __attribute__ ((aligned));
 
 	r = new_run_request();
 	g_assert(dbus_message_iter_init(msg, &iter));
@@ -180,6 +188,10 @@ filter(DBusConnection *con, DBusMessage *msg, void *user_data)
 	} else if (dbus_message_is_method_call(msg, "org.freedesktop.HalRunner", "Kill")) {
 		handle_kill(con, msg);
 		return DBUS_HANDLER_RESULT_HANDLED;
+	} else if (dbus_message_is_method_call(msg, "org.freedesktop.HalRunner", "Shutdown")) {
+		run_kill_all ();
+		exit (0);
+		return DBUS_HANDLER_RESULT_HANDLED;
 	} else if (dbus_message_is_method_call(msg, "org.freedesktop.HalRunner", "KillAll")) {
 		run_kill_all();
 		/* alwasy successfull */
