@@ -41,7 +41,9 @@
 #include <signal.h>
 #include <grp.h>
 #include <syslog.h>
-
+#include <sys/time.h>
+#include <sys/resource.h>
+#include <malloc.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -57,18 +59,13 @@
 #include "util.h"
 #include "hald_runner.h"
 #include "util_helper.h"
+#include "mmap_cache.h"
 
 static void delete_pid(void)
 {
 	unlink(HALD_PID_FILE);
 }
 
-/**
- * @defgroup HalDaemon HAL daemon
- * @brief The HAL daemon manages persistent device objects available through
- *        a D-BUS network API
- */
-
 static HalDeviceStore *global_device_list = NULL;
 
 static HalDeviceStore *temporary_device_list = NULL;
@@ -79,7 +76,7 @@ addon_terminated (HalDevice *device, guint32 exit_type,
 		  gint return_code, gchar **error,
 		  gpointer data1, gpointer data2)
 {
-	HAL_INFO (("in addon_terminated for udi=%s", device->udi));
+	HAL_INFO (("in addon_terminated for udi=%s", hal_device_get_udi (device)));
 
 	/* TODO: log to syslog - addons shouldn't just terminate, this is a bug with the addon */
 
@@ -101,26 +98,25 @@ gdl_store_changed (HalDeviceStore *store, HalDevice *device,
 		   gboolean is_added, gpointer user_data)
 {
 	if (is_added) {
-		GSList *addons;
+		HalDeviceStrListIter iter;
 
 		HAL_INFO (("Added device to GDL; udi=%s", hal_device_get_udi(device)));
 
-		if ((addons = hal_device_property_get_strlist (device, "info.addons")) != NULL) {
-			GSList *i;
+		for (hal_device_property_strlist_iter_init (device, "info.addons", &iter);
+		     hal_device_property_strlist_iter_is_valid (&iter);
+		     hal_device_property_strlist_iter_next (&iter)) {
+			const gchar *command_line;
+			gchar *extra_env[2] = {"HALD_ACTION=addon", NULL};
+			
+			command_line = hal_device_property_strlist_iter_get_value (&iter);
 
-			for (i = addons; i != NULL; i = g_slist_next (i)) {
-				const gchar *command_line;
-				gchar *extra_env[2] = {"HALD_ACTION=addon", NULL};
-
-				command_line = (const gchar *) i->data;
-				if (hald_runner_start(device, command_line, extra_env, addon_terminated, NULL, NULL)) {
-					HAL_INFO (("Started addon %s for udi %s", 
-						   command_line, hal_device_get_udi(device)));
-					hal_device_inc_num_addons (device);
-				} else {
-					HAL_ERROR (("Cannot start addon %s for udi %s", 
-						    command_line, hal_device_get_udi(device)));
-				}
+			if (hald_runner_start(device, command_line, extra_env, addon_terminated, NULL, NULL)) {
+				HAL_INFO (("Started addon %s for udi %s", 
+					   command_line, hal_device_get_udi(device)));
+				hal_device_inc_num_addons (device);
+			} else {
+				HAL_ERROR (("Cannot start addon %s for udi %s", 
+					    command_line, hal_device_get_udi(device)));
 			}
 		}
 	} else {
@@ -165,6 +161,32 @@ gdl_capability_added (HalDeviceStore *store, HalDevice *device,
 	/*hal_callout_capability (device, capability, TRUE)*/;
 }
 
+static void
+gdl_lock_acquired (HalDeviceStore *store, HalDevice *device, const char *lock_name, const char *lock_owner)
+{
+	if (hal_device_are_all_addons_ready (device)) {
+                if (strncmp (lock_name, "Global.", 7) == 0 && 
+                    strcmp (hal_device_get_udi (device), "/org/freedesktop/Hal/devices/computer") == 0) {
+                        manager_send_signal_interface_lock_acquired (lock_name + 7, lock_owner);
+                } else {
+                        device_send_signal_interface_lock_acquired (device, lock_name, lock_owner);
+                }
+	}
+}
+
+static void
+gdl_lock_released (HalDeviceStore *store, HalDevice *device, const char *lock_name, const char *lock_owner)
+{
+	if (hal_device_are_all_addons_ready (device)) {
+                if (strncmp (lock_name, "Global.", 7) == 0 && 
+                    strcmp (hal_device_get_udi (device), "/org/freedesktop/Hal/devices/computer") == 0) {
+                        manager_send_signal_interface_lock_released (lock_name + 7, lock_owner);
+                } else {
+                        device_send_signal_interface_lock_released (device, lock_name, lock_owner);
+                }
+	}
+}
+
 HalDeviceStore *
 hald_get_gdl (void)
 {
@@ -180,6 +202,15 @@ hald_get_gdl (void)
 		g_signal_connect (global_device_list,
 				  "device_capability_added",
 				  G_CALLBACK (gdl_capability_added), NULL);
+		g_signal_connect (global_device_list,
+				  "device_property_changed",
+				  G_CALLBACK (gdl_property_changed), NULL);
+		g_signal_connect (global_device_list,
+				  "device_lock_acquired",
+				  G_CALLBACK (gdl_lock_acquired), NULL);
+		g_signal_connect (global_device_list,
+				  "device_lock_released",
+				  G_CALLBACK (gdl_lock_released), NULL);
 	}
 
 	return global_device_list;
@@ -196,15 +227,10 @@ hald_get_tdl (void)
 	return temporary_device_list;
 }
 
-/**
- * @defgroup MainDaemon Basic functions
- * @ingroup HalDaemon
- * @brief Basic functions in the HAL daemon
- * @{
- */
-
-/** Print out program usage.
+/** 
+ * usage: 
  *
+ * Print out program usage.
  */
 static void
 usage ()
@@ -212,13 +238,16 @@ usage ()
 	fprintf (stderr, "\n" "usage : hald [--daemon=yes|no] [--verbose=yes|no] [--help]\n");
 	fprintf (stderr,
 		 "\n"
-		 "        --daemon=yes|no      Become a daemon\n"
-		 "        --verbose=yes|no     Print out debug (overrides HALD_VERBOSE)\n"
- 		 "        --use-syslog         Print out debug messages to syslog instead of stderr.\n"
-		 "                             Use this option to get debug messages if HAL runs as\n"
-		 "                             daemon.\n"
-		 "        --help               Show this information and exit\n"
-		 "        --version            Output version information and exit"
+		 "        --daemon=yes|no       Become a daemon\n"
+		 "        --verbose=yes|no      Print out debug (overrides HALD_VERBOSE)\n"
+		 "        --retain-privileges   Retain privileges (for debugging)\n"
+ 		 "        --use-syslog          Print out debug messages to syslog instead of\n"
+		 "                              stderr. Use this option to get debug messages\n"
+		 "                              if hald runs as a daemon.\n"
+		 "        --help                Show this information and exit\n"
+		 "        --version             Output version information and exit\n"
+		 "        --exit-after-probing  Exit when probing is complete. Useful only\n"
+		 "                              when profiling hald.\n"
 		 "\n"
 		 "The HAL daemon detects devices present in the system and provides the\n"
 		 "org.freedesktop.Hal service through the system-wide message bus provided\n"
@@ -231,9 +260,13 @@ usage ()
 /** If #TRUE, we will daemonize */
 static dbus_bool_t opt_become_daemon = TRUE;
 
+/** If #TRUE, we will retain privs */
+static dbus_bool_t opt_retain_privileges = FALSE;
+
 /** If #TRUE, we will spew out debug */
 dbus_bool_t hald_is_verbose = FALSE;
 dbus_bool_t hald_use_syslog = FALSE;
+dbus_bool_t hald_debug_exit_after_probing = FALSE;
 
 static int sigterm_unix_signal_pipe_fds[2];
 static GIOChannel *sigterm_iochn;
@@ -352,11 +385,13 @@ out:
 
 /*--------------------------------------------------------------------------------------------------*/
 
-/** Entry point for HAL daemon
+/**  
+ *  main:
+ *  @argc:               Number of arguments
+ *  @argv:               Array of arguments
+ *  Returns:             Exit code
  *
- *  @param  argc                Number of arguments
- *  @param  argv                Array of arguments
- *  @return                     Exit code
+ *  Entry point for HAL daemon
  */
 int
 main (int argc, char *argv[])
@@ -368,6 +403,23 @@ main (int argc, char *argv[])
 
 	openlog ("hald", LOG_PID, LOG_DAEMON);
 
+#ifdef HAVE_MALLOPT
+
+#define HAL_MMAP_THRESHOLD 100
+#define HAL_TRIM_THRESHOLD 100
+
+	/* We use memory in small chunks, thus optimize
+	   it this way.
+	 */
+	mallopt(M_MMAP_THRESHOLD, HAL_MMAP_THRESHOLD);
+	mallopt(M_TRIM_THRESHOLD, HAL_TRIM_THRESHOLD);
+
+#endif
+
+#ifdef HALD_MEMLEAK_DBG
+	/*g_mem_set_vtable (glib_mem_profiler_table);*/
+#endif
+
 	g_type_init ();
 
 	if (getenv ("HALD_VERBOSE"))
@@ -393,14 +445,17 @@ main (int argc, char *argv[])
 	g_strlcat (newpath, PACKAGE_SCRIPT_DIR, sizeof (newpath));
 
 	setenv ("PATH", newpath, TRUE);
+	
 
 	while (1) {
 		int c;
 		int option_index = 0;
 		const char *opt;
 		static struct option long_options[] = {
+			{"exit-after-probing", 0, NULL, 0},
 			{"daemon", 1, NULL, 0},
 			{"verbose", 1, NULL, 0},
+			{"retain-privileges", 0, NULL, 0},
 			{"use-syslog", 0, NULL, 0},
 			{"help", 0, NULL, 0},
 			{"version", 0, NULL, 0},
@@ -422,6 +477,8 @@ main (int argc, char *argv[])
 			} else if (strcmp (opt, "version") == 0) {
 				fprintf (stderr, "HAL package version: " PACKAGE_VERSION "\n");
 				return 0;
+			} else if (strcmp (opt, "exit-after-probing") == 0) {
+				hald_debug_exit_after_probing = TRUE;
 			} else if (strcmp (opt, "daemon") == 0) {
 				if (strcmp ("yes", optarg) == 0) {
 					opt_become_daemon = TRUE;
@@ -440,6 +497,8 @@ main (int argc, char *argv[])
 					usage ();
 					return 1;
 				}
+			} else if (strcmp (opt, "retain-privileges") == 0) {
+                                opt_retain_privileges = TRUE;
 			} else if (strcmp (opt, "use-syslog") == 0) {
                                 hald_use_syslog = TRUE;
 			}
@@ -467,10 +526,11 @@ main (int argc, char *argv[])
 	/*master_slave_setup ();
 	  sleep (100000000);*/
 
+
 	loop = g_main_loop_new (NULL, FALSE);
 
 	HAL_INFO ((PACKAGE_STRING));
-
+	
 	if (opt_become_daemon) {
 		int child_pid;
 		int dev_null_fd;
@@ -537,7 +597,6 @@ main (int argc, char *argv[])
 		HAL_INFO (("Will not daemonize"));
 	}
 
-
 	/* we need to do stuff when we are expected to terminate, thus
 	 * this involves looking for SIGTERM; UNIX signal handlers are
 	 * evil though, so set up a pipe to transmit the signal.
@@ -563,17 +622,35 @@ main (int argc, char *argv[])
 	/* set up the local dbus server */
 	if (!hald_dbus_local_server_init ())
 		return 1;
+
+	if (!hald_dbus_init_preprobe ()) {
+		return 1;
+	}
+
 	/* Start the runner helper daemon */
 	if (!hald_runner_start_runner ()) {
 		return 1;
 	}
 
-	drop_privileges(0);
+	/* initialize privileged operating system specific parts */
+	osspec_privileged_init ();
+
+	/* sometimes we don't want to drop root privs, for instance
+	   if we are profiling memory usage */
+	if (opt_retain_privileges == FALSE) {
+		drop_privileges(0);
+	}
+
+	hald_is_initialising = TRUE;
+
+	/* make sure our fdi rule cache is up to date */
+	di_cache_coherency_check();
 
 	/* initialize operating system specific parts */
 	osspec_init ();
 
-	hald_is_initialising = TRUE;
+	/* Init FDI files */
+	di_rules_init();
 
 	/* detect devices */
 	osspec_probe ();
@@ -588,13 +665,22 @@ main (int argc, char *argv[])
 extern int dbg_hal_device_object_delta;
 
 /* useful for valgrinding; see below */
+static gboolean
+my_shutdown2 (gpointer data)
+{
+	/*g_mem_profile ();*/
+	/*sleep (10000);*/
+	exit (1);
+	return FALSE;
+}
+
 static gboolean
 my_shutdown (gpointer data)
 {
 	HalDeviceStore *gdl;
 	
-	printf ("Num devices in TDL: %d\n", g_slist_length ((hald_get_tdl ())->devices));
-	printf ("Num devices in GDL: %d\n", g_slist_length ((hald_get_gdl ())->devices));
+	HAL_INFO (("Num devices in TDL: %d", g_slist_length ((hald_get_tdl ())->devices)));
+	HAL_INFO (("Num devices in GDL: %d", g_slist_length ((hald_get_gdl ())->devices)));
 	
 	gdl = hald_get_gdl ();
 next:
@@ -603,10 +689,19 @@ next:
 		hal_device_store_remove (gdl, d);
 		g_object_unref (d);
 		goto next;
-	}
-	
-	printf ("hal_device_object_delta = %d (should be zero)\n", dbg_hal_device_object_delta);
-	exit (1);
+	}	
+
+	HAL_INFO (("hal_device_object_delta = %d (should be zero)", dbg_hal_device_object_delta));
+
+	hald_dbus_local_server_shutdown ();
+	hald_runner_stop_runner ();
+
+	HAL_INFO (("Shutting down in five seconds"));
+	g_timeout_add (5 * 1000,
+		       my_shutdown2,
+		       NULL);
+
+	return FALSE;
 }
 #endif
 
@@ -618,6 +713,13 @@ osspec_probe_done (void)
 
 	HAL_INFO (("Device probing completed"));
 
+	if (hald_debug_exit_after_probing) {
+		HAL_INFO (("Exiting on user request (--exit-after-probing)"));
+		hald_runner_kill_all();
+		exit (0);
+	}
+
+
 	if (!hald_dbus_init ()) {
 		hald_runner_kill_all();
 		exit (1);
@@ -637,5 +739,3 @@ osspec_probe_done (void)
 #endif
 }
 
-
-/** @} */
