@@ -36,6 +36,7 @@
 #include "osspec_linux.h"
 
 #include "acpi.h"
+#include "device.h"
 
 enum {
 	ACPI_TYPE_BATTERY,
@@ -60,12 +61,14 @@ typedef struct ACPIDevHandler_s
 	HalDevice *(*add) (const gchar *acpi_path, HalDevice *parent, struct ACPIDevHandler_s *handler);
 	gboolean (*compute_udi) (HalDevice *d, struct ACPIDevHandler_s *handler);
 	gboolean (*remove) (HalDevice *d, struct ACPIDevHandler_s *handler);
-	gboolean (*refresh) (HalDevice *d, struct ACPIDevHandler_s *handler);
+	gboolean (*refresh) (HalDevice *d, struct ACPIDevHandler_s *handler, gboolean force_full_refresh);
 } ACPIDevHandler;
 
-/** Just sets the ac_adapter.present key when called
+/** 
+ *  ac_adapter_refresh_poll
+ *  @d:		valid ac_adaptor HalDevice
  *
- *  @param	d		valid ac_adaptor HalDevice
+ * Just sets the ac_adapter.present key when called 
  */
 static void
 ac_adapter_refresh_poll (HalDevice *d)
@@ -131,18 +134,24 @@ battery_refresh_poll (HalDevice *d)
 	reporting_rate = hal_device_property_get_int (d,
 					"battery.reporting.rate");
 
+	/*
+	 * ACPI gives out the special 'Ones' value for rate when it's unable
+	 * to calculate the true rate. We should set the rate zero, and wait
+	 * for the BIOS to stabilise.
+	 *
+	 * full details here: http://bugzilla.gnome.org/show_bug.cgi?id=348201
+	 */
+	if (reporting_rate == 0xffff) {
+		reporting_rate = 0;
+	}
+
 	/*
 	 * We are converting the unknown units into mWh because of ACPI's nature
 	 * of not having a standard "energy" unit.
 	 *
 	 * full details here: http://bugzilla.gnome.org/show_bug.cgi?id=309944
 	 */
-	if (reporting_unit && strcmp (reporting_unit, "mWh") == 0) {
-		/* units do not need conversion */
-		normalised_current = reporting_current;
-		normalised_lastfull = reporting_lastfull;
-		normalised_rate = reporting_rate;
-	} else if (reporting_unit && strcmp (reporting_unit, "mAh") == 0) {
+	if (reporting_unit && strcmp (reporting_unit, "mAh") == 0) {
 		/* convert mAh to mWh by multiplying by voltage.  due to the
 		 * general wonkiness of ACPI implementations, this is a lot
 		 * harder than it should have to be...
@@ -160,18 +169,20 @@ battery_refresh_poll (HalDevice *d)
 		if (design_voltage <= 0)
 			design_voltage = 1000; /* mV */
 
-		/* If the current voltage is unknown or greater than design,
-		 * then use design voltage.
+		/* If the current voltage is unknown, smaller than 50% of design voltage (fd.o #8593) 
+		 * or greater than design, then use design voltage.
 		 */
-		if (voltage <= 0 || voltage > design_voltage)
+		if (voltage < (design_voltage/2)  || voltage > design_voltage) {
+			HAL_DEBUG (("Current voltage is unknown, smaller than 50%% or greater than design"));
 			voltage = design_voltage;
+		}
 
 		normalised_current = (reporting_current * voltage) / 1000;
 		normalised_lastfull = (reporting_lastfull * voltage) / 1000;
 		normalised_rate = (reporting_rate * voltage) / 1000;
 	} else {
 		/*
-		 * handle as if mWh, which is the most common case.
+		 * handle as if mWh (which don't need conversion), which is the most common case.
 		 */
 		normalised_current = reporting_current;
 		normalised_lastfull = reporting_lastfull;
@@ -211,11 +222,13 @@ battery_refresh_poll (HalDevice *d)
 	hal_device_property_set_int (d, "battery.charge_level.last_full", normalised_lastfull);
 	hal_device_property_set_int (d, "battery.charge_level.rate", normalised_rate);
 
-	remaining_time = util_compute_time_remaining (d->udi, normalised_rate, normalised_current, normalised_lastfull,
-				hal_device_property_get_bool (d, "battery.rechargeable.is_discharging"),
-				hal_device_property_get_bool (d, "battery.rechargeable.is_charging"),
-				hal_device_property_get_bool (d, "battery.remaining_time.calculate_per_time"));
-	remaining_percentage = util_compute_percentage_charge (d->udi, normalised_current, normalised_lastfull);
+	remaining_time = util_compute_time_remaining (
+		hal_device_get_udi (d), 
+		normalised_rate, normalised_current, normalised_lastfull,
+		hal_device_property_get_bool (d, "battery.rechargeable.is_discharging"),
+		hal_device_property_get_bool (d, "battery.rechargeable.is_charging"),
+		hal_device_property_get_bool (d, "battery.remaining_time.calculate_per_time"));
+	remaining_percentage = util_compute_percentage_charge (hal_device_get_udi (d), normalised_current, normalised_lastfull);
 	/*
 	 * Only set keys if no error (signified with negative return value)
 	 * Scrict checking is needed to ensure that the values presented by HAL
@@ -233,13 +246,15 @@ battery_refresh_poll (HalDevice *d)
 		hal_device_property_remove (d, "battery.charge_level.percentage");
 }
 
-/** Recalculates the battery.reporting.last_full key as this may drift
- *  over time.
+/** 
+ *  battery_poll_infrequently:
+ *  @data:		Ignored
+ *  Returns:  		TRUE if we updated values
  *
- *  @param	data		Ignored
- *  @return			TRUE if we updated values
+ *  Recalculates the battery.reporting.last_full key as this may drift
+ *  over time. 
  *
- *  @note	This is called 120x less often than battery_refresh_poll
+ *  Note: This is called 120x less often than battery_refresh_poll
  */
 static gboolean
 battery_poll_infrequently (gpointer data) {
@@ -272,11 +287,14 @@ battery_poll_infrequently (gpointer data) {
 }
 
 
-/** Fallback polling method to refresh battery objects is plugged in
+/** 
+ *  acpi_poll_battery:
  *
  *  @return			TRUE
  *
- *  @note	This just calls battery_refresh_poll for each battery
+ * Fallback polling method to refresh battery objects is plugged in 
+ *
+ * Note: This just calls battery_refresh_poll for each battery
  */
 static gboolean
 acpi_poll_battery (void)
@@ -307,11 +325,14 @@ acpi_poll_battery (void)
 	return TRUE;
 }
 
-/** Fallback polling method to detect if the ac_adaptor is plugged in
+/** 
+ *  acpi_poll_acadap:
+ * 
+ *  Returns:			TRUE
  *
- *  @return			TRUE
+ *  Fallback polling method to detect if the ac_adaptor is plugged in 
  *
- *  @note	This just calls ac_adapter_refresh_poll for each ac_adapter
+ *  Note: This just calls ac_adapter_refresh_poll for each ac_adapter
  */
 static gboolean
 acpi_poll_acadap (void)
@@ -340,13 +361,16 @@ acpi_poll_acadap (void)
 	return TRUE;
 }
 
-/** Fallback polling method called every minute.
+/** 
+ *  acpi_poll:
+ *  @data:		Ignored
  *
- *  @param	data		Ignored
- *  @return			TRUE
+ *  Returns:		TRUE
  *
- *  @note	This just forces a poll refresh for *every* ac_adaptor
- *		and primary battery in the system.
+ *  Fallback polling method called every minute. 
+ *
+ *  Note: This just forces a poll refresh for *every* ac_adaptor
+ *        and primary battery in the system.
  */
 static gboolean
 acpi_poll (gpointer data)
@@ -361,7 +385,7 @@ acpi_poll (gpointer data)
 }
 
 static gboolean
-ac_adapter_refresh (HalDevice *d, ACPIDevHandler *handler)
+ac_adapter_refresh (HalDevice *d, ACPIDevHandler *handler, gboolean force_full_refresh)
 {
 	const char *path;
 	path = hal_device_property_get_string (d, "linux.acpi_path");
@@ -393,11 +417,13 @@ ac_adapter_refresh (HalDevice *d, ACPIDevHandler *handler)
 	return TRUE;
 }
 
-/** Removes all the possible battery.* keys.
+/** 
+ *  battery_refresh_remove:
+ *  @d:		Valid battery HalDevice
  *
- *  @param	d		Valid battery HalDevice
+ *  Removes all the possible battery.* keys. 
  *
- *  @note	Removing a key that doesn't exist is OK.
+ *  Note: Removing a key that doesn't exist is OK.
  */
 static void
 battery_refresh_remove (HalDevice *d)
@@ -439,10 +465,12 @@ battery_refresh_remove (HalDevice *d)
 	hal_device_property_remove (d, "battery.remaining_time");
 }
 
-/** Adds all the possible battery.* keys and does coldplug (slowpath)
- *  type calculations.
+/** 
+ *  battery_refresh_add:
+ *  @d:		Valid battery HalDevice
  *
- *  @param	d		Valid battery HalDevice
+ *  Adds all the possible battery.* keys and does coldplug (slowpath)
+ *  type calculations. 
  */
 static gboolean
 battery_refresh_add (HalDevice *d, const char *path)
@@ -460,7 +488,7 @@ battery_refresh_add (HalDevice *d, const char *path)
 	hal_util_set_string_elem_from_file (d, "battery.vendor", path, "info",
 					    "OEM info", 0, TRUE);
 	hal_util_set_string_elem_from_file (d, "battery.model", path, "info",
-					    "model number", 0, TRUE);
+					    "model number", G_MAXUINT, TRUE);
 	hal_util_set_string_elem_from_file (d, "battery.serial", path, "info",
 					    "serial number", 0, TRUE);
 	hal_util_set_string_elem_from_file (d, "battery.vendor", path, "info",
@@ -515,17 +543,7 @@ battery_refresh_add (HalDevice *d, const char *path)
 	reporting_gran1 = hal_device_property_get_int (d, "battery.reporting.granularity_1");
 	reporting_gran2 = hal_device_property_get_int (d, "battery.reporting.granularity_2");
 
-	if (reporting_unit && strcmp (reporting_unit, "mWh") == 0) {
-		/* do not scale */
-		hal_device_property_set_int (d, "battery.charge_level.design", reporting_design);
-		hal_device_property_set_int (d, "battery.charge_level.warning", reporting_warning);
-		hal_device_property_set_int (d, "battery.charge_level.low", reporting_low);
-		hal_device_property_set_int (d, "battery.charge_level.granularity_1", reporting_gran1);
-		hal_device_property_set_int (d, "battery.charge_level.granularity_2", reporting_gran2);
-
-		/* set unit */
-		hal_device_property_set_string (d, "battery.charge_level.unit", "mWh");
-	} else if (reporting_unit && strcmp (reporting_unit, "mAh") == 0) {
+	if (reporting_unit && strcmp (reporting_unit, "mAh") == 0) {
 		voltage_design = hal_device_property_get_int (d, "battery.voltage.design");
 	
 		/* If design voltage is unknown, use 1V. */
@@ -545,30 +563,26 @@ battery_refresh_add (HalDevice *d, const char *path)
 			(reporting_gran2 * voltage_design) / 1000);
 
 		/* set unit */
-		hal_device_property_set_string (d, "battery.charge_level.unit",
-			"mWh"); /* not mAh! */
+		hal_device_property_set_string (d, "battery.charge_level.unit", "mWh"); /* not mAh! */
 	} else {
 		/*
-		 * Some ACPI BIOS's do not report the unit,
-		 * so we'll assume they are mWh.
-		 * We will report the guessing with the
-		 * battery.charge_level.unit key.
+		 * Some ACPI BIOS's do not report the unit, so we'll assume they are mWh.
+		 * We will report the guessing with the battery.charge_level.unit key.
 		 */
-		hal_device_property_set_int (d,
-			"battery.charge_level.design", reporting_design);
-		hal_device_property_set_int (d,
-			"battery.charge_level.warning", reporting_warning);
-		hal_device_property_set_int (d,
-			"battery.charge_level.low", reporting_low);
-		hal_device_property_set_int (d,
-			"battery.charge_level.granularity_1", reporting_gran1);
-		hal_device_property_set_int (d,
-			"battery.charge_level.granularity_2", reporting_gran2);
+		hal_device_property_set_int (d, "battery.charge_level.design", reporting_design);
+		hal_device_property_set_int (d, "battery.charge_level.warning", reporting_warning);
+		hal_device_property_set_int (d, "battery.charge_level.low", reporting_low);
+		hal_device_property_set_int (d, "battery.charge_level.granularity_1", reporting_gran1);
+		hal_device_property_set_int (d, "battery.charge_level.granularity_2", reporting_gran2);
 
-		/* set "Unknown ACPI Unit" unit so we can debug */
-		HAL_WARNING (("Unknown ACPI Unit!"));
-		hal_device_property_set_string (d, "battery.charge_level.unit",
-			"unknown");
+		/* set unit */
+		if (reporting_unit && strcmp (reporting_unit, "mWh") == 0) {
+			hal_device_property_set_string (d, "battery.charge_level.unit", "mWh");
+		} else {
+			/* set "Unknown ACPI Unit" unit so we can debug */
+			HAL_WARNING (("Unknown ACPI Unit!"));
+			hal_device_property_set_string (d, "battery.charge_level.unit", "unknown");
+		}
 	}
 
 	/* set alarm if present */
@@ -582,8 +596,17 @@ battery_refresh_add (HalDevice *d, const char *path)
 	return TRUE;
 }
 
+/** 
+ *  battery_refresh:
+ *  @d:		Valid battery HalDevice
+ *  @handler:	The ACPIDevHandler.
+ *  
+ *  Returns:    TRUE/FALSE
+ *
+ *  Refresh the battery device information. 
+ */
 static gboolean
-battery_refresh (HalDevice *d, ACPIDevHandler *handler)
+battery_refresh (HalDevice *d, ACPIDevHandler *handler, gboolean force_full_refresh)
 {
 	const char *path;
 
@@ -613,7 +636,7 @@ battery_refresh (HalDevice *d, ACPIDevHandler *handler)
 		 * /proc/acpi/battery/BAT%d/[info|state] so don't read
 		 * static data that won't change
 		 */
-		if (!hal_device_has_property (d, "battery.vendor")) {
+		if (force_full_refresh || !hal_device_has_property (d, "battery.vendor")) {
 			/* battery has no information, so coldplug */
 			battery_refresh_add (d, path);
 		}
@@ -630,6 +653,14 @@ battery_refresh (HalDevice *d, ACPIDevHandler *handler)
 	return TRUE;
 }
 
+/** 
+ *  get_processor_model_name:
+ *  @proc_num:	Number of processor
+ *  
+ *  Returns:    Name of the processor model
+ *
+ *  Give the model name of the processor. 
+ */
 static gchar *
 get_processor_model_name (gint proc_num)
 {
@@ -684,7 +715,7 @@ get_processor_model_name (gint proc_num)
 }
 
 static gboolean
-processor_refresh (HalDevice *d, ACPIDevHandler *handler)
+processor_refresh (HalDevice *d, ACPIDevHandler *handler, gboolean force_full_refresh)
 {
 	const char *path;
 
@@ -718,7 +749,7 @@ processor_refresh (HalDevice *d, ACPIDevHandler *handler)
 }
 
 static gboolean
-fan_refresh (HalDevice *d, ACPIDevHandler *handler)
+fan_refresh (HalDevice *d, ACPIDevHandler *handler, gboolean force_full_refresh)
 {
 	const char *path;
 
@@ -740,13 +771,17 @@ fan_refresh (HalDevice *d, ACPIDevHandler *handler)
  * Using /proc/acpi/video does not work, this method is much more reliable.
  */
 static gboolean
-laptop_panel_refresh (HalDevice *d, ACPIDevHandler *handler)
+laptop_panel_refresh (HalDevice *d, ACPIDevHandler *handler, gboolean force_full_refresh)
 {
 	const char *path;
 	int acpi_type;
 	char *type = NULL;
 	char *desc = NULL;
 	int br_levels = -1;
+	
+	if ((hal_device_store_find (hald_get_gdl (),
+				    "/org/freedesktop/Hal/devices/computer_backlight")) != NULL)
+		return FALSE;
 
 	path = hal_device_property_get_string (d, "linux.acpi_path");
 	if (path == NULL)
@@ -775,9 +810,25 @@ laptop_panel_refresh (HalDevice *d, ACPIDevHandler *handler)
 		desc = "Sony LCD Panel";
 		br_levels = 8;
 	} else if (acpi_type == ACPI_TYPE_OMNIBOOK_DISPLAY) {
+		gchar *proc_lcd;
+		gchar proc_path[HAL_PATH_MAX];	
+		int current = -1;
+		int max = -1;
+
 		type = "omnibook";
 		desc = "Omnibook LCD Panel";
-		br_levels = 8;
+		/* 
+		 * There are different support brightness level, depending on 
+		 * the hardware and the kernel module version.
+		 */
+		snprintf (proc_path, sizeof (proc_path), "%s/%s", get_hal_proc_path (), "omnibook");
+		proc_lcd = hal_util_grep_file(proc_path, "lcd", "LCD brightness:", FALSE);
+		proc_lcd = g_strstrip (proc_lcd);
+		if (sscanf (proc_lcd, "%d (max value: %d)", &current, &max) == 2) {	
+			br_levels = max + 1;
+		} else {	
+			br_levels = 11;
+		}
 	} else if (acpi_type == ACPI_TYPE_SONYPI_DISPLAY) {
 		type = "sonypi";
 		desc = "Sony LCD Panel";
@@ -805,7 +856,7 @@ laptop_panel_refresh (HalDevice *d, ACPIDevHandler *handler)
 }
 
 static gboolean
-button_refresh (HalDevice *d, ACPIDevHandler *handler)
+button_refresh (HalDevice *d, ACPIDevHandler *handler, gboolean force_full_refresh)
 {
 	const char *path;
 	gchar *parent_path;
@@ -840,10 +891,12 @@ button_refresh (HalDevice *d, ACPIDevHandler *handler)
 	return TRUE;
 }
 
-/** Synthesizes a *specific* acpi object.
+/** 
+ *  acpi_synthesize_item:
+ *  @fullpath:		The ACPI path, e.g. "/proc/acpi/battery/BAT1"
+ *  @acpi_type:		The type of device, e.g. ACPI_TYPE_BATTERY
  *
- *  @param	fullpath	The ACPI path, e.g. "/proc/acpi/battery/BAT1"
- *  @param	acpi_type	The type of device, e.g. ACPI_TYPE_BATTERY
+ *  Synthesizes a *specific* acpi object.
  */
 static void
 acpi_synthesize_item (const gchar *fullpath, int acpi_type)
@@ -858,12 +911,14 @@ acpi_synthesize_item (const gchar *fullpath, int acpi_type)
 	hotplug_event_enqueue (hotplug_event);
 }
 
-/** Synthesizes generic acpi objects, i.e. registers all the objects of type
- *  into HAL. This lets us have more than one type of device e.g. BATx
- *  in the same battery class.
+/** 
+ *  acpi_synthesize:
+ *  @path:		The ACPI path, e.g. "/proc/acpi/battery"
+ *  @acpi_type:		The type of device, e.g. ACPI_TYPE_BATTERY
  *
- *  @param	path		The ACPI path, e.g. "/proc/acpi/battery"
- *  @param	acpi_type	The type of device, e.g. ACPI_TYPE_BATTERY
+ *  Synthesizes generic acpi objects, i.e. registers all the objects of type
+ *  into HAL. This lets us have more than one type of device e.g. BATx
+ *  in the same battery class. 
  */
 static void
 acpi_synthesize (const gchar *path, int acpi_type)
@@ -893,6 +948,8 @@ acpi_synthesize (const gchar *path, int acpi_type)
 				snprintf (_path, sizeof (_path), "%s/acpi/button/lid", get_hal_proc_path ());
 				if ( strcmp (path, _path) == 0 )
 					is_laptop = TRUE;
+			} else if (_have_sysfs_lid_button) {
+				is_laptop = TRUE;
 			}
 		}
 		/* if there is a battery bay or LID, this is a laptop -> set the formfactor */
@@ -915,12 +972,14 @@ acpi_synthesize (const gchar *path, int acpi_type)
 	g_dir_close (dir);
 }
 
-/** If {procfs_path}/acpi/{vendor}/{display} is found, then add the
- *  LaptopPanel device.
+/** 
+ *  acpi_synthesize_display:
+ *  @vendor:		The vendor name, e.g. sony
+ *  @display:		The *possible* name of the brightness file
+ *  @method:		The HAL enumerated type.
  *
- *  @param	vendor		The vendor name, e.g. sony
- *  @param	display		The *possible* name of the brightness file
- *  @param	method		The HAL enumerated type.
+ *  If {procfs_path}/acpi/{vendor}/{display} is found, then add the
+ *  LaptopPanel device. 
  */
 static void
 acpi_synthesize_display (char *vendor, char *display, int method)
@@ -937,7 +996,10 @@ acpi_synthesize_display (char *vendor, char *display, int method)
 
 static int sonypi_irq_list[] = { 11, 10, 9, 6, 5 };
 
-/** Synthesizes a sonypi object.
+/** 
+ *  acpi_synthesize_sonypi_display:
+ *
+ *  Synthesizes a sonypi object.
  */
 static void
 acpi_synthesize_sonypi_display (void)
@@ -978,11 +1040,14 @@ acpi_synthesize_sonypi_display (void)
 	g_free (path);
 }
 
-/** Scan the data structures exported by the kernel and add hotplug
- *  events for adding ACPI objects.
+/** 
+ *  acpi_synthesize_hotplug_events:
  *
- *  @return			TRUE if, and only if, ACPI capabilities
- *				were detected
+ *  Returns:		TRUE if, and only if, ACPI capabilities
+ *			were detected
+ *
+ *  Scan the data structures exported by the kernel and add hotplug
+ *  events for adding ACPI objects. 
  */
 gboolean
 acpi_synthesize_hotplug_events (void)
@@ -990,7 +1055,7 @@ acpi_synthesize_hotplug_events (void)
 	HalDevice *computer;
 	gchar path[HAL_PATH_MAX];
 
-	if (!g_file_test ("/proc/acpi/info", G_FILE_TEST_EXISTS))
+	if (!g_file_test ("/proc/acpi/", G_FILE_TEST_IS_DIR))
 		return FALSE;
 
 	if ((computer = hal_device_store_find (hald_get_gdl (), "/org/freedesktop/Hal/devices/computer")) == NULL &&
@@ -1001,8 +1066,16 @@ acpi_synthesize_hotplug_events (void)
 
 	/* Set appropriate properties on the computer object */
 	hal_device_property_set_string (computer, "power_management.type", "acpi");
-	hal_util_set_string_elem_from_file (computer, "power_management.acpi.linux.version",
-					    "/proc/acpi", "info", "version", 0, FALSE);
+	if (g_file_test ("/proc/acpi/info", G_FILE_TEST_EXISTS)) {
+		hal_util_set_string_elem_from_file (computer, "power_management.acpi.linux.version",
+						    "/proc/acpi", "info", "version", 0, FALSE);
+	} else {
+		gchar *firmware_path;
+		firmware_path = g_strdup_printf ("%s/firmware/acpi", get_hal_sysfs_path ());
+		hal_util_set_string_elem_from_file (computer, "power_management.acpi.linux.version",
+						    firmware_path, "info", "version", 0, FALSE);
+		g_free (firmware_path);
+	}
 
 	/* collect batteries */
 	snprintf (path, sizeof (path), "%s/acpi/battery", get_hal_proc_path ());
@@ -1065,16 +1138,38 @@ acpi_generic_add (const gchar *acpi_path, HalDevice *parent, ACPIDevHandler *han
 	hal_device_property_set_string (d, "linux.acpi_path", acpi_path);
 	hal_device_property_set_int (d, "linux.acpi_type", handler->acpi_type);
 	if (parent != NULL)
-		hal_device_property_set_string (d, "info.parent", parent->udi);
+		hal_device_property_set_string (d, "info.parent", hal_device_get_udi (parent));
 	else
 		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
-	if (handler->refresh == NULL || !handler->refresh (d, handler)) {
+	if (handler->refresh == NULL || !handler->refresh (d, handler, FALSE)) {
 		g_object_unref (d);
 		d = NULL;
 	}
 	return d;
 }
 
+static HalDevice *
+acpi_button_add (const gchar *acpi_path, HalDevice *parent, ACPIDevHandler *handler)
+{
+	char *parent_path;
+	const char *button_type;
+
+	parent_path = hal_util_get_parent_path (acpi_path);
+	button_type = hal_util_get_last_element (parent_path);
+	if (strcmp (button_type, "lid") == 0 && _have_sysfs_lid_button)
+		goto out;
+	else if (strcmp (button_type, "power") == 0 && _have_sysfs_power_button)
+		goto out;
+	else if (strcmp (button_type, "sleep") == 0 && _have_sysfs_sleep_button)
+		goto out;
+
+	g_free (parent_path);
+	return acpi_generic_add (acpi_path, parent, handler);
+
+out:
+	return NULL;
+}
+
 static gboolean
 acpi_generic_compute_udi (HalDevice *d, ACPIDevHandler *handler)
 {
@@ -1176,7 +1271,7 @@ static ACPIDevHandler acpidev_handler_laptop_panel_sonypi = {
 
 static ACPIDevHandler acpidev_handler_button = {
 	.acpi_type   = ACPI_TYPE_BUTTON,
-	.add         = acpi_generic_add,
+	.add         = acpi_button_add,
 	.compute_udi = acpi_generic_compute_udi,
 	.refresh     = button_refresh,
 	.remove      = acpi_generic_remove
@@ -1211,7 +1306,7 @@ acpi_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer userdata2)
 {
 	void *end_token = (void *) userdata1;
 
-	HAL_INFO (("Add callouts completed udi=%s", d->udi));
+	HAL_INFO (("Add callouts completed udi=%s", hal_device_get_udi (d)));
 
 	/* Move from temporary to global device store */
 	hal_device_store_remove (hald_get_tdl (), d);
@@ -1225,7 +1320,7 @@ acpi_callouts_remove_done (HalDevice *d, gpointer userdata1, gpointer userdata2)
 {
 	void *end_token = (void *) userdata1;
 
-	HAL_INFO (("Remove callouts completed udi=%s", d->udi));
+	HAL_INFO (("Remove callouts completed udi=%s", hal_device_get_udi (d)));
 
 	if (!hal_device_store_remove (hald_get_gdl (), d)) {
 		HAL_WARNING (("Error removing device"));
@@ -1260,11 +1355,6 @@ hotplug_event_begin_add_acpi (const gchar *acpi_path, int acpi_type, HalDevice *
 			/* Add to temporary device store */
 			hal_device_store_add (hald_get_tdl (), d);
 
-			/* Merge properties from .fdi files */
-			di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
-			di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
-
-			
 			/* Compute UDI */
 			if (!handler->compute_udi (d, handler)) {
 				hal_device_store_remove (hald_get_tdl (), d);
@@ -1272,6 +1362,10 @@ hotplug_event_begin_add_acpi (const gchar *acpi_path, int acpi_type, HalDevice *
 				return;
 			}
 
+			/* Merge properties from .fdi files */
+			di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
+			di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
+			
 			/* Run callouts */
 			hal_util_callout_device_add (d, acpi_callouts_add_done, end_token, NULL);
 			return;
@@ -1327,11 +1421,11 @@ acpi_rescan_device (HalDevice *d)
 
 		handler = acpi_handlers[i];
 		if (handler->acpi_type == acpi_type) {
-			return handler->refresh (d, handler);
+			return handler->refresh (d, handler, TRUE);
 		}
 	}
 
-	HAL_WARNING (("Didn't find a rescan handler for udi %s", d->udi));
+	HAL_WARNING (("Didn't find a rescan handler for udi %s", hal_device_get_udi (d)));
 	return TRUE;
 }
 
