@@ -40,7 +40,7 @@
 #include "../../logger.h"
 
 #ifdef HAVE_POLKIT
-#include <libpolkit/libpolkit.h>
+#include <libpolkit.h>
 #endif
 
 #define MAX_LINE_SIZE				255
@@ -147,8 +147,16 @@ gboolean write_line(const char *filename, const char *fmt, ...)
 	return TRUE;
 }
 
-/** reads one line from filename, splits it by delim and returns a two
- * dimension array of strings or NULL on error */
+/** 
+ * read_line_str_split:
+ * @filename:	Filename
+ * @delim:      The delimiter to split
+ *
+ * Returns:     The splitted line as array
+ * 
+ * Reads one line from filename, splits it by delim and returns a two
+ * dimension array of strings or NULL on error 
+ */
 static gchar **read_line_str_split(char *filename, gchar *delim)
 {
 	gchar	line[MAX_LINE_SIZE];
@@ -177,8 +185,17 @@ static gchar **read_line_str_split(char *filename, gchar *delim)
 	return l;
 }
 
-/** reads one line from filename, splits its integers by delim and stores
- * all items in the given list */
+/** 
+ * read_line_int_split:
+ * @filename:	Filename
+ * @delim:      The delimiter to split
+ * @list:       List with the splitted integers 
+ *
+ * Returns:     TRUE/FALSE
+ *
+ * Reads one line from filename, splits its integers by delim and stores
+ * all items in the given list 
+ */
 gboolean read_line_int_split(char *filename, gchar *delim, GSList **list)
 {
 	gchar	**l;
@@ -186,6 +203,9 @@ gboolean read_line_int_split(char *filename, gchar *delim, GSList **list)
 
 	l = read_line_str_split(filename, delim);
 
+        if (l == NULL)
+                return FALSE;
+	
 	for (i = 0; l[i] != NULL; i++) {
 		int value = atoi(l[i]);
 		*list = g_slist_append(*list, GINT_TO_POINTER(value));
@@ -313,7 +333,7 @@ gboolean write_governor(char *new_governor, int cpu_id)
 		goto Out;
 
 	governor_file = g_strdup_printf(SYSFS_GOVERNOR_FILE, cpu_id); 
-	HAL_DEBUG(("Trying ot write governor %s", new_governor));
+	HAL_DEBUG(("Trying to write governor %s", new_governor));
 
 	if (!write_line(governor_file, "%s", new_governor)) {
 		ret = FALSE;
@@ -383,7 +403,7 @@ static gboolean ondemand_set_consider_nice(void *data, gboolean consider)
 
 	consider_file = g_strdup_printf(ONDEMAND_IGNORE_NICE_LOAD_FILE, iface->base_cpu); 
 
-        if(!write_line(consider_file, "%u", consider)){
+        if(!write_line(consider_file, "%u", !consider)){
                 HAL_WARNING(("Could not set ignore_nice_load to: %u kHz; %s", consider,
 			     strerror(errno)));
 		g_free(consider_file);
@@ -410,7 +430,7 @@ static gboolean ondemand_get_consider_nice(void)
 	}
 	g_free(governor_file);
 
-	return consider;
+	return !consider;
 }
 
 static gboolean ondemand_init(struct ondemand_interface *iface, GSList *cores)
@@ -437,9 +457,17 @@ static void ondemand_free(void *data)
 
 /********************* main interface *********************/
 
-/** sets the performance for all cpufreq objects
+/**
+ * set_performance: 
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @performance:	performance value to set
+ *
+ * Returns: 		TRUE/FALSE
  *
  * @raises NoSuitableGoveror
+ *
+ * sets the performance for all cpufreq objects
  */
 static gboolean set_performance(DBusConnection *connection, DBusMessage *message,
 				int performance)
@@ -478,9 +506,17 @@ static gboolean set_performance(DBusConnection *connection, DBusMessage *message
 	return TRUE;
 }
 
-/** sets the performance for all cpufreq objects
+/** 
+ * get_performance:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @performance:	pointer to return the current performance value
+ *
+ * Returns: 		TRUE/FALSE
  *
  * @raises (NoSuitableGoveror|GeneralError)
+ * 
+ * sets the performance for all cpufreq objects
  */
 static gboolean get_performance(DBusConnection *connection, DBusMessage *message,
 				int *performance)
@@ -520,9 +556,17 @@ static gboolean get_performance(DBusConnection *connection, DBusMessage *message
 	return TRUE;
 }
 
-/** sets the performance for all cpufreq objects
+/** 
+ * set_consider_nice:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @consider:		TRUE if should set consider nice
+ *
+ * Returns: 		TRUE/FALSE
  *
  * @raises NoSuitableGoveror
+ * 
+ * sets consider nice to all cpufreq opjects. 
  */
 static gboolean set_consider_nice(DBusConnection *connection, DBusMessage *message,
 				  gboolean consider)
@@ -542,9 +586,17 @@ static gboolean set_consider_nice(DBusConnection *connection, DBusMessage *messa
 	return TRUE;
 }
 
-/** sets the performance for all cpufreq objects
+/** 
+ * get_consider_nice:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @consider:		pointer to return the current state of consider nice
+ *
+ * Returns: 		TRUE/FALSE
  *
  * @raises NoSuitableGoveror
+ * 
+ * Gets consider nice state. 
  */
 static gboolean get_consider_nice(DBusConnection *connection, DBusMessage *message,
 				  int *consider)
@@ -563,9 +615,17 @@ static gboolean get_consider_nice(DBusConnection *connection, DBusMessage *messa
 	return TRUE;
 }
 
-/** stores a list of all available governors in the given list.
+/** 
+ * get_available_governors:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @governors:		to return the current available governors.
+ *
+ * Returns: 		TRUE/FALSE
  *
  * @raises GeneralError
+ * 
+ * Gets the names of all availabe governors. 
  */
 static gboolean get_available_governors(DBusConnection *connection, DBusMessage *message,
 					gchar ***governors)
@@ -585,9 +645,17 @@ static gboolean get_available_governors(DBusConnection *connection, DBusMessage
 	return TRUE;
 }
 
-/** sets a governor for all cpufreq objects
+/** 
+ * set_governors:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @governor:		name of the governor to set
  *
- * @raises (GeneralError|UnknownGovernor|GovernorInitFailed)
+ * Returns: 		TRUE/FALSE
+ *
+ * @raises (GeneralError|UnknownGovernor|GovernorInitFailed) 
+ * 
+ * sets a governor for all cpufreq objects. 
  */
 static gboolean set_governors(DBusConnection *connection, DBusMessage *message,
 				       const char *governor)
@@ -715,9 +783,17 @@ static gboolean set_governors(DBusConnection *connection, DBusMessage *message,
 	return TRUE;
 }
 
-/** gets the current governor which is set for all cpufreq objects
+/** 
+ * get_governors:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @governor:		pointer to the name of the current set governor
  *
- * @raises GeneralError
+ * Returns: 		TRUE/FALSE
+ *
+ * @raises GeneralError 
+ * 
+ * gets the current governor which is set for all cpufreq objects
  */
 static gboolean get_governors(DBusConnection *connection, DBusMessage *message,
 			   char *governor)
@@ -744,8 +820,17 @@ static gboolean get_governors(DBusConnection *connection, DBusMessage *message,
 
 /********************* DBus stuff *********************/
 
-/** raises the NoSuitableGovernor error with the given method in the
- * detail field */
+/** 
+ * dbus_raise_no_suitable_governor:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @method:		name of the method
+ *
+ * Returns: 		TRUE/FALSE
+ *
+ * Raises the NoSuitableGovernor error with the given method in the
+ * detail field
+ */
 static gboolean dbus_raise_no_suitable_governor(DBusConnection *connection,
 						DBusMessage *message,
 						char *method)
@@ -756,8 +841,17 @@ static gboolean dbus_raise_no_suitable_governor(DBusConnection *connection,
 				method);
 }
 
-/** raises the GovernorInitFailed error with the given governor in the
- * detail field */
+/** 
+ * dbus_raise_governor_init_failed:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @governor:		name of the governor
+ *
+ * Returns: 		TRUE/FALSE
+ *
+ * Raises the GovernorInitFailed error with the given governor in the
+ * detail field
+ */
 static gboolean dbus_raise_governor_init_failed(DBusConnection *connection,
 						DBusMessage *message,
 						char *governor)
@@ -768,7 +862,18 @@ static gboolean dbus_raise_governor_init_failed(DBusConnection *connection,
 				governor);
 }
 
-/** raises the given error_name with the format in the detail field */
+/** 
+ * dbus_raise_error:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @error_name:		name of the error
+ * @format:             to format the error message
+ * @...:		more args
+ *
+ * Returns: 		TRUE/FALSE
+ *
+ * Raises the given error_name with the format in the detail field
+ */
 static gboolean dbus_raise_error(DBusConnection *connection, DBusMessage *message,
 				 const char *error_name, char *format, ...)
 {
@@ -803,7 +908,16 @@ static gboolean dbus_raise_error(DBusConnection *connection, DBusMessage *messag
 }
 
 #ifdef HAVE_POLKIT
-/** checks if caller of message possesses the CPUFREQ_POLKIT_PRIVILGE */
+/** 
+ * dbus_is_privileged:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @error:		the error
+ *
+ * Returns: 		TRUE if the caller is privileged
+ *
+ * checks if caller of message possesses the CPUFREQ_POLKIT_PRIVILGE 
+ */
 static gboolean dbus_is_privileged(DBusConnection *connection, DBusMessage *message,
 				   DBusError *error)
 {
@@ -815,7 +929,7 @@ static gboolean dbus_is_privileged(DBusConnection *connection, DBusMessage *mess
 	gboolean		out_is_allowed;
 	gboolean		out_is_temporary;
 	LibPolKitResult		res;
-    
+
 	connection_new = dbus_bus_get(DBUS_BUS_SYSTEM, error);
 	if (dbus_error_is_set(error)) {
 		dbus_raise_error(connection, message, CPUFREQ_ERROR_GENERAL,
@@ -831,9 +945,13 @@ static gboolean dbus_is_privileged(DBusConnection *connection, DBusMessage *mess
 	}
 
 	caller_dbus_name = dbus_message_get_sender(message);
+	if (caller_dbus_name == NULL) {
+		dbus_raise_error(connection, message, CPUFREQ_ERROR_GENERAL,
+				 "Cannot get D-Bus connection name of caller");
+		goto Error;
+	}
+		
 	caller_unix_user = dbus_bus_get_unix_user(connection_new, caller_dbus_name, error);
-	HAL_DEBUG(("Connection name of caller: %s", caller_dbus_name));
-	HAL_DEBUG(("Unix user id of caller: %ld", caller_unix_user));
 	if (dbus_error_is_set(error)) {
 		dbus_raise_error(connection, message, CPUFREQ_ERROR_GENERAL,
 				 "Cannot get unix user of caller");
@@ -841,6 +959,9 @@ static gboolean dbus_is_privileged(DBusConnection *connection, DBusMessage *mess
 		goto Error;
 	}
 
+	HAL_DEBUG(("Connection name of caller: %s", caller_dbus_name));
+	HAL_DEBUG(("Unix user id of caller: %ld", caller_unix_user));
+
 	caller_unix_user_str = g_strdup_printf("%ld", caller_unix_user);
 	res = libpolkit_is_uid_allowed_for_privilege(polctx,
 						     caller_dbus_name,
@@ -874,7 +995,17 @@ Error:
 }
 #endif
 
-/** sends a reply to message with the given data and its dbus_type */
+/** 
+ * dbus_send_reply:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @type:               the type of data param
+ * @data:		data to send
+ *
+ * Returns: 		TRUE/FALSE
+ *
+ * sends a reply to message with the given data and its dbus_type 
+ */
 static gboolean dbus_send_reply(DBusConnection *connection, DBusMessage *message,
 				int dbus_type, void *data)
 {
@@ -898,7 +1029,16 @@ static gboolean dbus_send_reply(DBusConnection *connection, DBusMessage *message
 	return TRUE;
 }
 
-/** sends a reply to message appending a list of strings */
+/** 
+ * dbus_send_reply:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @list:               list to send
+ *
+ * Returns: 		TRUE/FALSE
+ *
+ * sends a reply to message appending a list of strings 
+ */
 static gboolean dbus_send_reply_strlist(DBusConnection *connection, DBusMessage *message,
 					gchar **list)
 {
@@ -934,9 +1074,17 @@ static gboolean dbus_send_reply_strlist(DBusConnection *connection, DBusMessage
 	return TRUE;
 }
 
-/** gets one argument from message with the given dbus_type and stores it
- * in arg
+/** 
+ * dbus_get_argument:
+ * @connection:		connection to D-Bus
+ * @message:		Message
+ * @dbus_error:         the D-Bus error
+ * @type:               the type of arg param
+ * @arg:		the value to get from the message
  *
+ * Returns: 		TRUE/FALSE
+ *
+ * gets one argument from message with the given dbus_type and stores it in arg
  */
 static gboolean dbus_get_argument(DBusConnection *connection, DBusMessage *message,
 				  DBusError *dbus_error, int dbus_type, void *arg)
@@ -952,9 +1100,30 @@ static gboolean dbus_get_argument(DBusConnection *connection, DBusMessage *messa
 	return TRUE;
 }
 
-/** dbus filter function
+static DBusHandlerResult dbus_filter_function_local(DBusConnection *connection,
+						    DBusMessage *message,
+						    void *user_data)
+{
+	if (dbus_message_is_signal(message, DBUS_INTERFACE_LOCAL,
+				   "Disconnected")) {
+		HAL_DEBUG(("DBus daemon disconnected. Trying to reconnect..."));
+		dbus_connection_unref(connection);
+		g_timeout_add(5000, (GSourceFunc)dbus_init_local, NULL);
+	}
+	return DBUS_HANDLER_RESULT_HANDLED;
+}
+
+/** 
+ * dbus_filter_function:
+ * @connection:		connection to D-Bus
+ * @message:		message
+ * @user_data:          pointer to the data
  *
+ * Returns: 		the result
+ * 
  * @raises UnknownMethod
+ *
+ * D-Bus filter function
  */
 static DBusHandlerResult dbus_filter_function(DBusConnection *connection,
 					      DBusMessage *message,
@@ -1043,13 +1212,6 @@ static DBusHandlerResult dbus_filter_function(DBusConnection *connection,
 			dbus_send_reply_strlist(connection, message, governors);
 		g_strfreev(governors);
 
-	} else if (dbus_message_is_signal(message, DBUS_INTERFACE_LOCAL,
-					  "Disconnected")) {
-		HAL_DEBUG(("DBus daemon disconnected. Trying to reconnect..."));
-		dbus_connection_close(connection);
-		dbus_connection_unref(connection);
-		g_timeout_add(5000, (GSourceFunc)dbus_init, NULL);
-
 	} else {
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
@@ -1070,7 +1232,27 @@ static gboolean is_supported(void)
 	return TRUE;
 }
 
-/** returns FALSE on success because it's used as a callback */
+/* returns FALSE on success because it's used as a callback */
+gboolean dbus_init_local(void)
+{
+	DBusConnection	*dbus_connection;
+	DBusError	dbus_error;
+
+	dbus_error_init(&dbus_error);
+
+	dbus_connection = dbus_bus_get(DBUS_BUS_SYSTEM, &dbus_error);
+	if (dbus_error_is_set(&dbus_error)) {
+		HAL_WARNING(("Cannot get D-Bus connection"));
+		return TRUE;
+	}
+
+	dbus_connection_setup_with_g_main(dbus_connection, NULL);
+	dbus_connection_add_filter(dbus_connection, dbus_filter_function_local,
+				   NULL, NULL);
+	dbus_connection_set_exit_on_disconnect(dbus_connection, 0);
+	return FALSE;
+}
+
 gboolean dbus_init(void)
 {
 	DBusError	dbus_error;
@@ -1090,12 +1272,10 @@ gboolean dbus_init(void)
 		goto Error;
 	}
 
-	dbus_error_init (&dbus_error);
 	if (!libhal_device_addon_is_ready (halctx, udi, &dbus_error)) {
 		goto Error;
 	}
 
-
 	if (!libhal_device_claim_interface(halctx, udi,
 		"org.freedesktop.Hal.Device.CPUFreq", 
 		"    <method name=\"SetCPUFreqGovernor\">\n"
@@ -1132,16 +1312,21 @@ gboolean dbus_init(void)
 	libhal_device_add_capability(halctx,
 				     "/org/freedesktop/Hal/devices/computer",
 				     "cpufreq_control",
-				     NULL);
+				     &dbus_error);
 
+	if (dbus_error_is_set(&dbus_error)) {
+		HAL_WARNING(("Cannot add capability cpufreq_control: %s", dbus_error.message));
+		goto Error;
+	}
+	
 	dbus_connection_setup_with_g_main(dbus_connection, NULL);
 	dbus_connection_add_filter(dbus_connection, dbus_filter_function, NULL, NULL);
 	dbus_connection_set_exit_on_disconnect(dbus_connection, 0);
-	return FALSE;
+	return TRUE;
 
 Error:
 	dbus_error_free(&dbus_error);
-	return TRUE;
+	return FALSE;
 }
 /********************* DBus end *********************/
 
@@ -1179,7 +1364,7 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
-	if (dbus_init())
+	if (!dbus_init() || dbus_init_local())
 		exit(EXIT_FAILURE);
 
 	gmain = g_main_loop_new(NULL, FALSE);
