@@ -32,10 +32,12 @@
 #include <mntent.h>
 #include <stdint.h>
 #include <stdlib.h>
+#include <dirent.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
@@ -106,7 +108,7 @@ blockdev_compute_udi (HalDevice *d)
 		} else {
 			hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
 					      "%s_storage", 
-					      hal_device_property_get_string (d, "storage.physical_device"));
+					      hal_device_property_get_string (d, "storage.originating_device"));
 		}
 	}
 
@@ -122,7 +124,7 @@ blockdev_callouts_add_done (HalDevice *d, gpointer userdata1, gpointer userdata2
 {
 	void *end_token = (void *) userdata1;
 
-	HAL_INFO (("Add callouts completed udi=%s", d->udi));
+	HAL_INFO (("Add callouts completed udi=%s", hal_device_get_udi (d)));
 
 	/* Move from temporary to global device store */
 	hal_device_store_remove (hald_get_tdl (), d);
@@ -136,7 +138,7 @@ blockdev_callouts_remove_done (HalDevice *d, gpointer userdata1, gpointer userda
 {
 	void *end_token = (void *) userdata1;
 
-	HAL_INFO (("Remove callouts completed udi=%s", d->udi));
+	HAL_INFO (("Remove callouts completed udi=%s", hal_device_get_udi (d)));
 
 	if (!hal_device_store_remove (hald_get_gdl (), d)) {
 		HAL_WARNING (("Error removing device"));
@@ -168,7 +170,6 @@ blockdev_refresh_mount_state (HalDevice *d)
 	dev_t devt = makedev(0, 0);
 	GSList *volumes = NULL;
 	GSList *volume;
-        GSList *autofs_mounts = NULL;
 
 	/* open /proc/mounts */
 	g_snprintf (buf, sizeof (buf), "%s/mounts", get_hal_proc_path ());
@@ -188,6 +189,9 @@ blockdev_refresh_mount_state (HalDevice *d)
 	/* loop over /proc/mounts */
 	while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
 		struct stat statbuf;
+		gboolean use_device_name_for_match;
+
+		/*HAL_INFO ((" * /proc/mounts contain dev %s - type %s", mnt.mnt_fsname, mnt.mnt_type));*/
 
 		/* If this is a nfs mount (fstype == 'nfs') ignore the mount. Reason:
 		 *  1. we don't list nfs devices in HAL
@@ -197,48 +201,60 @@ blockdev_refresh_mount_state (HalDevice *d)
 		if (strcmp(mnt.mnt_type, "nfs") == 0)
 			continue;
 
-		/* If this is an autofs mount (fstype == 'autofs') 
-		 * store the mount in a list for later use. 
-		 * On mounts managed by autofs accessing files below the mount
-		 * point cause the mount point to be remounted after an 
-		 * unmount.  We keep the list so we do not check for
-		 * the .created-by-hal file on mounts under autofs mount points
-		 */
-		if (strcmp(mnt.mnt_type, "autofs") == 0) {
-			char *mnt_dir;
-
-			if (mnt.mnt_dir[strlen (mnt.mnt_dir) - 1] != '/')
-				mnt_dir = g_strdup_printf ("%s/", mnt.mnt_dir);
-			else
-				mnt_dir = g_strdup (mnt.mnt_dir);
-
-			autofs_mounts = g_slist_append (autofs_mounts,
-							mnt_dir);
-
-
-			continue;
+		use_device_name_for_match = FALSE;
+		/* get major:minor of special device file */
+		if (stat (mnt.mnt_fsname, &statbuf) != 0) {
+			/* DING DING DING... device node may have been deleted by udev 
+			 * but device is still mounted and we haven't processed the uevent
+			 * for that deletion from udev.. 
+			 * 
+			 * So in this case... fall back to comparing on device names 
+			 * rather than pretending the device is not mounted as that's
+			 * what will happen if we just skip this /proc/mounts entry.
+			 *
+			 * The reason it's nicer to compare on major:minor is that
+			 * /proc/mounts is broken - it contains the *device name*
+			 * passed to mount(2) which in some cases may be a symlink
+			 * (on many distros it's common to see /proc/mounts contain
+			 * /dev/root as the device for /). Sigh...
+			 */
+			use_device_name_for_match = TRUE;
+		} else {
+			if (major (statbuf.st_rdev) == 0)
+				continue;
 		}
 
-		/* check the underlying device of the mount point */
-		if (stat (mnt.mnt_dir, &statbuf) != 0)
-			continue;
-		if (major(statbuf.st_dev) == 0)
-			continue;
-
-		/*HAL_INFO (("* found mounts dev %s (%i:%i)", mnt.mnt_fsname, major(statbuf.st_dev), minor(statbuf.st_dev)));*/
+		/*HAL_INFO (("* found mounts dev %s (%i:%i)", mnt.mnt_fsname, major (statbuf.st_rdev), minor (statbuf.st_rdev)));*/
 		/* match against all hal volumes */
 		for (volume = volumes; volume != NULL; volume = g_slist_next (volume)) {
 			HalDevice *dev;
+			gboolean is_match;
 
+			is_match = FALSE;
 			dev = HAL_DEVICE (volume->data);
-			major = hal_device_property_get_int (dev, "block.major");
-			if (major == 0)
-				continue;
-			minor = hal_device_property_get_int (dev, "block.minor");
-			devt = makedev(major, minor);
-			/*HAL_INFO (("  match %s (%i:%i)", hal_device_get_udi (dev), major, minor));*/
 
-			if (statbuf.st_dev == devt) {
+			if (use_device_name_for_match) {
+				const char *device_name;
+
+				device_name = hal_device_property_get_string (dev, "block.device");
+				if (device_name == NULL)
+					continue;
+
+				if (strcmp (device_name, mnt.mnt_fsname) == 0)
+					is_match = TRUE;
+			} else {
+				major = hal_device_property_get_int (dev, "block.major");
+				if (major == 0)
+					continue;
+				minor = hal_device_property_get_int (dev, "block.minor");
+				devt = makedev (major, minor);
+				/*HAL_INFO (("  match %s (%i:%i)", hal_device_get_udi (dev), major, minor));*/
+
+				if (statbuf.st_rdev == devt)
+					is_match = TRUE;
+			}
+
+			if (is_match) {
 				/* found entry for this device in /proc/mounts */
 				device_property_atomic_update_begin ();
 				hal_device_property_set_bool (dev, "volume.is_mounted", TRUE);
@@ -247,7 +263,7 @@ blockdev_refresh_mount_state (HalDevice *d)
 				hal_device_property_set_string (dev, "volume.mount_point", mnt.mnt_dir);
 				device_property_atomic_update_end ();
 				/*HAL_INFO (("  set %s to be mounted at %s (%s)",
-					   hal_device_get_udi (dev), mnt.mnt_dir,
+				  hal_device_get_udi (dev), mnt.mnt_dir,
 					   hasmntopt (&mnt, MNTOPT_RO) ? "ro" : "rw"));*/
 				volumes = g_slist_delete_link (volumes, volume);
 				break;
@@ -255,9 +271,6 @@ blockdev_refresh_mount_state (HalDevice *d)
 		}
 	}
 
-	g_slist_foreach (autofs_mounts, (GFunc) g_free, NULL);
-	g_slist_free (autofs_mounts);
-
 	/* all remaining volumes are not mounted */
 	for (volume = volumes; volume != NULL; volume = g_slist_next (volume)) {
 		HalDevice *dev;
@@ -270,7 +283,7 @@ blockdev_refresh_mount_state (HalDevice *d)
 		 * lost... it is merely delayed...
 		 */
 		if (device_is_executing_method (dev, "org.freedesktop.Hal.Device.Volume", "Unmount")) {
-			HAL_INFO (("/proc/mounts tells that %s is unmounted - waiting for Unmount() to complete to change mount state", dev->udi));
+			HAL_INFO (("/proc/mounts tells that %s is unmounted - waiting for Unmount() to complete to change mount state", hal_device_get_udi (dev)));
 		} else {
 			char *mount_point;
 
@@ -326,7 +339,7 @@ generate_fakevolume_hotplug_event_add_for_storage_device (HalDevice *d)
 
 	hotplug_event = g_new0 (HotplugEvent, 1);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
-	hotplug_event->type = HOTPLUG_EVENT_SYSFS;
+	hotplug_event->type = HOTPLUG_EVENT_SYSFS_BLOCK;
 	g_strlcpy (hotplug_event->sysfs.subsystem, "block", sizeof (hotplug_event->sysfs.subsystem));
 	g_strlcpy (hotplug_event->sysfs.sysfs_path, fake_sysfs_path, sizeof (hotplug_event->sysfs.sysfs_path));
 	if (device_file != NULL)
@@ -336,7 +349,6 @@ generate_fakevolume_hotplug_event_add_for_storage_device (HalDevice *d)
 	hotplug_event->sysfs.net_ifindex = -1;
 
 	hotplug_event_enqueue (hotplug_event);
-	hotplug_event_process_queue ();
 }
 
 static void 
@@ -586,19 +598,16 @@ canonicalize_filename (gchar *filename)
 static char *
 resolve_symlink (const char *file)
 {
-	GError *error;
 	char *dir;
-	char *link;
+	gchar link[HAL_PATH_MAX];
 	char *f;
 	char *f1;
 
 	f = g_strdup (file);
-
+	memset(link, 0, HAL_PATH_MAX);
 	while (g_file_test (f, G_FILE_TEST_IS_SYMLINK)) {
-		link = g_file_read_link (f, &error);
-		if (link == NULL) {
-			g_warning ("Cannot resolve symlink %s: %s", f, error->message);
-			g_error_free (error);
+		if(readlink(f, link, HAL_PATH_MAX - 1) < 0) {
+			g_warning ("Cannot resolve symlink %s: %s", f, strerror(errno));
 			g_free (f);
 			f = NULL;
 			goto out;
@@ -607,7 +616,6 @@ resolve_symlink (const char *file)
 		dir = g_path_get_dirname (f);
 		f1 = g_strdup_printf ("%s/%s", dir, link);
 		g_free (dir);
-		g_free (link);
 		g_free (f);
 		f = f1;
 	}
@@ -663,36 +671,21 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 
 	/* OK, no parent... it might a device-mapper device => check slaves/ subdir in sysfs */
 	if (parent == NULL && !is_partition && !is_fakevolume) {
-		GDir *dir;
-		GError *err = NULL;
+		DIR * dir;
+		struct dirent *dp;
 		char path[HAL_PATH_MAX];
 
-
-		/* sleep one second since device mapper needs additional
-		 * time before the device file is ready
-		 *
-		 * this is a hack and will only affect device mapper block
-		 * devices. It can go away once the kernel emits a "changed"
-		 * event for the device file (this is about to go upstream)
-		 * and we can depend on a released kernel with this feature.
-		 */
-		if (strncmp (hal_util_get_last_element (sysfs_path), "dm-", 3) == 0) {
-			HAL_INFO (("Waiting 1000ms to wait for device mapper to be ready", path));
-			usleep (1000 * 1000);
-		}
-
 		g_snprintf (path, HAL_PATH_MAX, "%s/slaves", sysfs_path);
 		HAL_INFO (("Looking in %s", path));
-		if ((dir = g_dir_open (path, 0, &err)) == NULL) {
-			HAL_WARNING (("Unable to open %s: %s", path, err->message));
-			g_error_free (err);
+
+		if ((dir = opendir (path)) == NULL) {
+			HAL_WARNING (("Unable to open %s: %s", path, strerror(errno)));
 		} else {
-			const char *f;
-			while (((f = g_dir_read_name (dir)) != NULL) && (parent == NULL)) {
+			while (((dp = readdir (dir)) != NULL) && (parent == NULL)) {
 				char *link;
 				char *target;
 
-				link = g_strdup_printf ("%s/%s", path, f);
+				link = g_strdup_printf ("%s/%s", path, dp->d_name);
 				target = resolve_symlink (link);
 				HAL_INFO ((" %s -> %s", link, target));
 
@@ -720,8 +713,8 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 							HAL_INFO ((" slave_volume_stordev_udi='%s'!", slave_volume_stordev_udi));
 							parent = hal_device_store_find (hald_get_gdl (), slave_volume_stordev_udi);
 							if (parent != NULL) {
-								HAL_INFO ((" parent='%s'!", parent->udi));
-								hal_device_property_set_string (d, "volume.crypto_luks.clear.backing_volume", slave_volume->udi);
+								HAL_INFO ((" parent='%s'!", hal_device_get_udi (parent)));
+								hal_device_property_set_string (d, "volume.crypto_luks.clear.backing_volume", hal_device_get_udi (slave_volume));
 								is_device_mapper = TRUE;
 							}
 						}
@@ -729,7 +722,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 				}
 				g_free (target);
 			}
-			g_dir_close (dir);
+			closedir(dir);
 			HAL_INFO (("Done looking in %s", path));
 		}
 		
@@ -747,8 +740,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	}
 
 	hal_device_property_set_string (d, "linux.sysfs_path", sysfs_path);
-	hal_device_property_set_string (d, "linux.sysfs_path_device", sysfs_path);
-	hal_device_property_set_string (d, "info.parent", parent->udi);
+	hal_device_property_set_string (d, "info.parent", hal_device_get_udi (parent));
 	hal_device_property_set_int (d, "linux.hotplug_type", HOTPLUG_EVENT_SYSFS_BLOCK);
 
 	hal_device_property_set_string (d, "block.device", device_file);
@@ -762,8 +754,8 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 	hal_device_property_set_int (d, "block.minor", minor);
 	hal_device_property_set_bool (d, "block.is_volume", is_partition || is_device_mapper || is_fakevolume);
 
-	if (hal_device_has_property(parent, "info.bus") &&
-		(strcmp(hal_device_property_get_string(parent, "info.bus"), "platform") == 0) &&
+	if (hal_device_has_property(parent, "info.subsystem") &&
+		(strcmp(hal_device_property_get_string(parent, "info.subsystem"), "platform") == 0) &&
 		(sscanf(hal_device_property_get_string(parent, "platform.id"), "floppy.%d", &floppy_num) == 1)) {
 		/* for now, just cheat here for floppy drives */
 
@@ -778,7 +770,8 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		hal_device_property_set_string (d, "info.vendor", "");
 		hal_device_property_set_string (d, "info.product", "PC Floppy Drive");
 		hal_device_property_set_string (d, "storage.drive_type", "floppy");
-		hal_device_property_set_string (d, "storage.physical_device", parent->udi);
+		hal_device_property_set_string (d, "storage.originating_device", hal_device_get_udi (parent));
+		hal_device_property_set_string (d, "storage.physical_device", hal_device_get_udi (parent));
 		hal_device_property_set_bool (d, "storage.removable", TRUE);
 		hal_device_property_set_bool (d, "storage.removable.media_available", FALSE);
 		hal_device_property_set_bool (d, "storage.hotpluggable", FALSE);
@@ -843,7 +836,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		/* walk up the device chain to find the physical device, 
 		 * start with our parent. On the way, optionally pick up
 		 * the scsi if it exists */
-		udi_it = parent->udi;
+		udi_it = hal_device_get_udi (parent);
 		while (udi_it != NULL) {
 			HalDevice *d_it;
 
@@ -855,10 +848,16 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 
 			/* Find device */
 			d_it = hal_device_store_find (hald_get_gdl (), udi_it);
-			g_assert (d_it != NULL);
+			if (d_it == NULL) { 
+				d_it = hal_device_store_find (hald_get_tdl (), udi_it);
+				if (d_it == NULL) {
+					HAL_WARNING (("Could not get device '%s' from gdl or tdl.", udi_it));
+					goto error;
+				}
+			}
 
-			/* Check info.bus */
-			if ((bus = hal_device_property_get_string (d_it, "info.bus")) != NULL) {
+			/* Check info.subsystem */
+			if ((bus = hal_device_property_get_string (d_it, "info.subsystem")) != NULL) {
 				if (strcmp (bus, "scsi") == 0) {
 					scsidev = d_it;
 					physdev = d_it;
@@ -916,6 +915,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 			goto error;
 		}
 
+		hal_device_property_set_string (d, "storage.originating_device", physdev_udi);
 		hal_device_property_set_string (d, "storage.physical_device", physdev_udi);
 
 		if (!hal_util_get_int_from_file (sysfs_path, "removable", (gint *) &is_removable, 10)) {
@@ -943,7 +943,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		 */
 		hal_device_property_set_bool (d, "storage.media_check_enabled", is_removable);
 
-		parent_bus = hal_device_property_get_string (parent, "info.bus");
+		parent_bus = hal_device_property_get_string (parent, "info.subsystem");
 		if (parent_bus == NULL) {
 			HAL_INFO (("parent_bus is NULL - wrong parent?"));
 			goto error;
@@ -1057,7 +1057,7 @@ hotplug_event_begin_add_blockdev (const gchar *sysfs_path, const gchar *device_f
 		 * VOLUMES
 		 *
 		 ************************/
-		hal_device_property_set_string (d, "block.storage_device", parent->udi);
+		hal_device_property_set_string (d, "block.storage_device", hal_device_get_udi (parent));
 
 		/* defaults */
 		hal_device_property_set_string (d, "storage.model", "");
@@ -1163,7 +1163,7 @@ force_unmount_cb (HalDevice *d, guint32 exit_type,
 {
 	void *end_token = (void *) data1;
 
-	HAL_INFO (("force_unmount_cb for udi='%s', exit_type=%d, return_code=%d", d->udi, exit_type, return_code));
+	HAL_INFO (("force_unmount_cb for udi='%s', exit_type=%d, return_code=%d", hal_device_get_udi (d), exit_type, return_code));
 
 	if (exit_type == HALD_RUN_SUCCESS && error != NULL && 
 	    error[0] != NULL && error[1] != NULL) {
@@ -1181,7 +1181,7 @@ force_unmount_cb (HalDevice *d, guint32 exit_type,
 
 }
 
-static void
+static gboolean
 force_unmount (HalDevice *d, void *end_token)
 {
 	const char *device_file;
@@ -1198,7 +1198,7 @@ force_unmount (HalDevice *d, void *end_token)
 		extra_env[0] = "HAL_METHOD_INVOKED_BY_UID=0";
 		extra_env[1] = NULL;
 		
-		HAL_INFO (("force_unmount for udi='%s'", d->udi));
+		HAL_INFO (("force_unmount for udi='%s'", hal_device_get_udi (d)));
 		syslog (LOG_NOTICE, "forcibly attempting to lazy unmount %s as enclosing drive was disconnected", device_file);
 		
 		unmount_stdin = "lazy\n";
@@ -1211,28 +1211,10 @@ force_unmount (HalDevice *d, void *end_token)
 					0,
 					force_unmount_cb,
 					end_token, NULL);
-
-/*
-		char *cleanup_stdin;
-		char *extra_env[2];
-		
-		HAL_INFO (("Cleaning up directory '%s' since it was created by HAL's Mount()", mount_point));
-		
-		extra_env[0] = g_strdup_printf ("HALD_CLEANUP=%s", mount_point);
-		extra_env[1] = NULL;
-		cleanup_stdin = "\n";
-		
-		hald_runner_run_method (dev, 
-					"hal-storage-cleanup-mountpoint", 
-					extra_env, 
-					cleanup_stdin, TRUE,
-					0,
-					cleanup_mountpoint_cb,
-					g_strdup (mount_point), NULL);
-*/
+		return TRUE;
+	} else {
+		return FALSE;
 	}
-
-
 }
 
 void
@@ -1258,7 +1240,7 @@ hotplug_event_begin_remove_blockdev (const gchar *sysfs_path, void *end_token)
 		if (fakevolume != NULL) {
 			HotplugEvent *hotplug_event;
 			HAL_INFO (("Storage device with a fakevolume is going away; "
-				   "synthesizing hotplug rem for fakevolume %s", fakevolume->udi));
+				   "synthesizing hotplug rem for fakevolume %s", hal_device_get_udi (fakevolume)));
 			hotplug_event = blockdev_generate_remove_hotplug_event (fakevolume);
 			if (hotplug_event != NULL) {
 				/* push synthesized event at front of queue and repost this event... this is such that
@@ -1276,8 +1258,13 @@ hotplug_event_begin_remove_blockdev (const gchar *sysfs_path, void *end_token)
 		
 		/* if we're mounted, then do a lazy unmount so the system can gracefully recover */
 		if (hal_device_property_get_bool (d, "volume.is_mounted")) {
-			force_unmount (d, end_token);
+			if (!force_unmount (d, end_token)) {
+				/* this wasn't mounted by us... carry on */
+				HAL_INFO (("device at sysfs_path %s is mounted, but not by HAL", sysfs_path));
+				hal_util_callout_device_remove (d, blockdev_callouts_remove_done, end_token, NULL);
+			}
 		} else {
+			HAL_INFO (("device at sysfs_path %s is not mounted", sysfs_path));
 			hal_util_callout_device_remove (d, blockdev_callouts_remove_done, end_token, NULL);
 		}
 	}
@@ -1318,11 +1305,11 @@ block_rescan_storage_done (HalDevice *d, guint32 exit_type,
 		if (fakevolume != NULL) {
 			/* generate hotplug event to remove the fakevolume */
 			HotplugEvent *hotplug_event;
-			HAL_INFO (("Media removal detected; synthesizing hotplug rem for fakevolume %s", fakevolume->udi));
+			HAL_INFO (("Media removal detected; synthesizing hotplug rem for fakevolume %s", 
+				   hal_device_get_udi (fakevolume)));
 			hotplug_event = blockdev_generate_remove_hotplug_event (fakevolume);
 			if (hotplug_event != NULL) {
 				hotplug_event_enqueue (hotplug_event);
-				hotplug_event_process_queue ();
 			}
 		}
 	}
@@ -1338,11 +1325,11 @@ blockdev_rescan_device (HalDevice *d)
 
 	ret = FALSE;
 
-	HAL_INFO (("Entering, udi=%s", d->udi));
+	HAL_INFO (("Entering, udi=%s", hal_device_get_udi (d)));
 
 	/* This only makes sense on storage devices */
 	if (hal_device_property_get_bool (d, "block.is_volume")) {
-		HAL_INFO (("No action on volumes", d->udi));
+		HAL_INFO (("No action on volumes", hal_device_get_udi (d)));
 		goto out;
 	}
 
