@@ -37,7 +37,7 @@ gboolean hal_util_get_driver_name (const char *sysfs_path, gchar *driver_name);
 
 gboolean hal_util_set_driver (HalDevice *d, const char *property_name, const char *sysfs_path);
 
-HalDevice *hal_util_find_closest_ancestor (const gchar *sysfs_path);
+gboolean hal_util_find_known_parent (const gchar *sysfs_path, HalDevice **parent, gchar **parent_path);
 
 
 #endif /* OSSPEC_LINUX_H */
