@@ -131,14 +131,14 @@ battery_refresh (HalDevice *d, PMUDevHandler *handler)
 
 		/* TODO: could read some pmu file? */
 		remaining_time = util_compute_time_remaining (
-					d->udi,
+					hal_device_get_udi (d),
 					hal_device_property_get_int (d, "battery.charge_level.rate"),
 					current,
 					last_full,
 					hal_device_property_get_bool (d, "battery.rechargeable.is_discharging"),
 					hal_device_property_get_bool (d, "battery.rechargeable.is_charging"),
 					hal_device_property_get_bool (d, "battery.remaining_time.calculate_per_time"));
-		remaining_percentage = util_compute_percentage_charge (d->udi, current, last_full);
+		remaining_percentage = util_compute_percentage_charge (hal_device_get_udi (d), current, last_full);
 		/*
 		 * Only set keys if no error (signified with negative return value)
 		 * Scrict checking is needed to ensure that the values presented by HAL
@@ -203,15 +203,23 @@ lid_button_refresh (HalDevice *d, PMUDevHandler *handler)
 	return TRUE;
 }
 
-/** Refreshes a laptop screen connected to a PMU controller.
- *  This is much simpler than ACPI as we have a *standard* ioctl to use.
+/** 
+ *  laptop_panel_refresh:
+ *  @d:			The hal device
+ *  @handler:		What to do
  *
- *  @param	d		The hal device
- *  @param	handler		What to do
+ *  Returns: 		TRUE/FALSE as result of the operation
+ *
+ *  Refreshes a laptop screen connected to a PMU controller.
+ *  This is much simpler than ACPI as we have a *standard* ioctl to use. 
  */
 static gboolean
 laptop_panel_refresh (HalDevice *d, PMUDevHandler *handler)
 {
+	if ((hal_device_store_find (hald_get_gdl (),
+				    "/org/freedesktop/Hal/devices/computer_backlight")) != NULL)
+		return FALSE;
+
 	hal_device_property_set_string (d, "info.category", "laptop_panel");
 	hal_device_property_set_string (d, "info.product", "Apple Laptop Panel");
 
@@ -290,10 +298,12 @@ pmu_poll (gpointer data)
 	return TRUE;
 }
 
-/** Synthesizes a *specific* PMU object.
+/** 
+ *  pmu_synthesize_item:
+ *  @fullpath:		The PMU path, e.g. "/dev/pmu/info"
+ *  @pmu_type:		The type of device, e.g. PMU_TYPE_BATTERY
  *
- *  @param	fullpath	The PMU path, e.g. "/dev/pmu/info"
- *  @param	pmu_type	The type of device, e.g. PMU_TYPE_BATTERY
+ *  Synthesizes a *specific* PMU object. 
  */
 static void
 pmu_synthesize_item (const gchar *fullpath, int pmu_type)
@@ -308,11 +318,13 @@ pmu_synthesize_item (const gchar *fullpath, int pmu_type)
 	hotplug_event_enqueue (hotplug_event);
 }
 
-/** Scan the data structures exported by the kernel and add hotplug
- *  events for adding PMU objects.
+/**
+ *  pmu_synthesize_hotplug_events  
  *
- *  @param                      TRUE if, and only if, PMU capabilities
- *                              were detected
+ *  Returns:              TRUE if, and only if, PMU capabilities were detected
+ *
+ *  Scan the data structures exported by the kernel and add hotplug
+ *  events for adding PMU objects.
  */
 gboolean
 pmu_synthesize_hotplug_events (void)
@@ -381,9 +393,6 @@ pmu_synthesize_hotplug_events (void)
 		/* Add Laptop Panel */
 		snprintf (path, sizeof (path), "%s/pmu/info", get_hal_proc_path ());
 		pmu_synthesize_item (path, PMU_TYPE_LAPTOP_PANEL);
-
-		/* If the machine has got battery bays then this is a laptop. */
-		hal_device_property_set_string (computer, "system.formfactor", "laptop");
 	}
 
 	/* setup timer for things that we need to poll */
@@ -403,7 +412,7 @@ pmu_generic_add (const gchar *pmu_path, HalDevice *parent, PMUDevHandler *handle
 	hal_device_property_set_string (d, "linux.pmu_path", pmu_path);
 	hal_device_property_set_int (d, "linux.pmu_type", handler->pmu_type);
 	if (parent != NULL)
-		hal_device_property_set_string (d, "info.parent", parent->udi);
+		hal_device_property_set_string (d, "info.parent", hal_device_get_udi (parent));
 	else
 		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
 	if (handler->refresh == NULL || !handler->refresh (d, handler)) {
@@ -502,12 +511,6 @@ hotplug_event_begin_add_pmu (const gchar *pmu_path, int pmu_type, HalDevice *par
 			/* Add to temporary device store */
 			hal_device_store_add (hald_get_tdl (), d);
 
-			/* Merge properties from .fdi files */
-			di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
-			di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
-
-			/* TODO: Run callouts */
-			
 			/* Compute UDI */
 			if (!handler->compute_udi (d, handler)) {
 				hal_device_store_remove (hald_get_tdl (), d);
@@ -515,6 +518,12 @@ hotplug_event_begin_add_pmu (const gchar *pmu_path, int pmu_type, HalDevice *par
 				goto out;
 			}
 
+			/* Merge properties from .fdi files */
+			di_search_and_merge (d, DEVICE_INFO_TYPE_INFORMATION);
+			di_search_and_merge (d, DEVICE_INFO_TYPE_POLICY);
+
+			/* TODO: Run callouts */
+			
 			/* Move from temporary to global device store */
 			hal_device_store_remove (hald_get_tdl (), d);
 			hal_device_store_add (hald_get_gdl (), d);
@@ -583,7 +592,7 @@ pmu_rescan_device (HalDevice *d)
 		}
 	}
 
-	HAL_WARNING (("Didn't find a rescan handler for udi %s", d->udi));
+	HAL_WARNING (("Didn't find a rescan handler for udi %s", hal_device_get_udi (d)));
 
 out:
 	return ret;
