@@ -39,6 +39,7 @@
 #include <string.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+#include <signal.h>
 
 #include <glib.h>
 #include <libvolume_id.h>
@@ -285,13 +286,21 @@ out:
 	lseek (fd, 0, SEEK_SET);
 }
 
+static char *device_file;
+
+static void
+handle_sigterm (int value)
+{
+	HAL_ERROR (("Timed out probing %s - broken device driver?", device_file));
+	exit (1);
+}
+
 int 
 main (int argc, char *argv[])
 {
 	int fd;
 	int ret;
 	char *udi;
-	char *device_file;
 	LibHalContext *ctx = NULL;
 	DBusError error;
 	char *parent_udi;
@@ -309,10 +318,11 @@ main (int argc, char *argv[])
 	dbus_bool_t should_probe_for_fs;
 	dbus_uint64_t vol_probe_offset = 0;
 	LibHalChangeSet *cs;
+	gboolean disc_may_have_data;
 	fd = -1;
 
-
 	cs = NULL;
+	disc_may_have_data = FALSE;
 
 	/* hook in our debug into libvolume_id */
 	volume_id_log_fn = vid_log;
@@ -360,6 +370,9 @@ main (int argc, char *argv[])
 
 	HAL_DEBUG(("Doing probe-volume for %s\n", device_file));
 
+	/* set up signal handler in case I/O to the device takes too long and the runner kills us */
+	signal (SIGTERM, handle_sigterm);
+
 	fd = open (device_file, O_RDONLY);
 	if (fd < 0)
 		goto out;
@@ -421,8 +434,19 @@ main (int argc, char *argv[])
 			libhal_changeset_set_property_bool (cs, "volume.disc.is_blank", TRUE);
 			/* set the volume size to 0 if disc is blank and not as 4 from BLKGETSIZE64 */
 			libhal_changeset_set_property_int (cs, "volume.block_size", 0);
-			HAL_DEBUG(("Disc in %s is blank", device_file));
-			should_probe_for_fs = FALSE;
+			HAL_DEBUG(("Disc in %s probably blank", device_file));
+			/* 
+			 * blank discs normally return 0x0800 (probably for the TOC) - if it's larger,
+			 * it probably means that the drive/firmware is bad and CDROM_DISC_STATUS lies.
+			 * In this case, actually try to probe for a file system. See RH #186334 for
+			 * details: https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=186334
+			 */
+			if (vol_size > 0x800) {
+				should_probe_for_fs = TRUE;
+				disc_may_have_data = TRUE;
+			} else {
+				should_probe_for_fs = FALSE;
+			}
 			break;
 			
 		default:		/* should never see this */
@@ -439,6 +463,12 @@ main (int argc, char *argv[])
 		HAL_DEBUG(("get_disc_type returned 0x%02x", type));
 		if (type != -1) {
 			switch (type) {
+			case 0x03: /* Magneto-Optical disk with sector erase capability */
+			case 0x05: /* Advance Storage – Magneto-Optical */
+				libhal_changeset_set_property_string (cs, "volume.disc.type", "mo");
+				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
+			case 0x04: /* Magneto Optical write once */
+				libhal_changeset_set_property_string (cs, "volume.disc.type", "mo");
 			case 0x08: /* CD-ROM */
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "cd_rom");
 				break;
@@ -460,9 +490,6 @@ main (int argc, char *argv[])
 				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
 				break;
 			case 0x13: /* DVD-RW Restricted Overwrite */
-				libhal_changeset_set_property_string (cs, "volume.disc.type", "dvd_rw");
-				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
-				break;
 			case 0x14: /* DVD-RW Sequential */
 				libhal_changeset_set_property_string (cs, "volume.disc.type", "dvd_rw");
 				libhal_changeset_set_property_bool (cs, "volume.disc.is_rewritable", TRUE);
@@ -481,8 +508,6 @@ main (int argc, char *argv[])
                           	libhal_changeset_set_property_string (cs, "volume.disc.type", "bd_rom");
 				break;
 			case 0x41: /* BD-R Sequential */
-                          	libhal_changeset_set_property_string (cs, "volume.disc.type", "bd_r");
-				break;
 			case 0x42: /* BD-R Random */
                           	libhal_changeset_set_property_string (cs, "volume.disc.type", "bd_r");
 				break;
@@ -586,8 +611,25 @@ main (int argc, char *argv[])
 		/* probe for file system */
 		vid = volume_id_open_fd (fd);
 		if (vid != NULL) {
-			if (volume_id_probe_all (vid, vol_probe_offset , vol_size) == 0) {
+			int vid_ret;
+			HAL_INFO (("invoking volume_id_probe_all, offset=%d, size=%d", vol_probe_offset, vol_size));
+			vid_ret = volume_id_probe_all (vid, vol_probe_offset , vol_size);
+			HAL_INFO (("volume_id_probe_all returned %d", vid_ret));
+
+			if (vid_ret != 0 && is_disc && vol_probe_offset != 0) {
+				/* Some cd-rom drives report the offset of the session in the cd's TOC
+				 * wrong.  Fallback to probing at offset 0, just to be sure */
+				HAL_INFO (("invoking volume_id_probe_all, offset=0, size=%d", vol_size));
+				vid_ret = volume_id_probe_all (vid, 0 , vol_size);
+				HAL_INFO (("volume_id_probe_all returned %d", vid_ret));
+			}
+
+			if (vid_ret == 0) {
 				set_volume_id_values(ctx, udi, cs, vid);
+				if (disc_may_have_data) {
+					libhal_changeset_set_property_bool (cs, "volume.disc.is_blank", FALSE);
+					libhal_changeset_set_property_bool (cs, "volume.disc.has_data", TRUE);
+				}
 			} else {
 				libhal_changeset_set_property_string (cs, "info.product", "Volume");
 			}
