@@ -67,7 +67,7 @@
 #define MOUNT_TYPE_OPT	"-F"
 #else
 #define MOUNT		"/bin/mount"
-#define MOUNT_OPTIONS	"noexec,nosuid,nodev"
+#define MOUNT_OPTIONS	"nosuid,nodev"
 #define MOUNT_TYPE_OPT	"-t"
 #endif
 
@@ -153,7 +153,6 @@ permission_denied_privilege (const char *privilege, const char *uid)
 }
 #endif
 
-
 /* borrowed from gtk/gtkfilesystemunix.c in GTK+ on 02/23/2006 */
 static void
 canonicalize_filename (gchar *filename)
@@ -272,7 +271,9 @@ volume_findby (LibHalContext *hal_ctx, const char *property, const char *value)
 	dbus_error_init (&error);
 	if ((hal_udis = libhal_manager_find_device_string_match (hal_ctx, property, 
 								 value, &num_hal_udis, &error)) == NULL) {
-		LIBHAL_FREE_DBUS_ERROR (&error);
+		if (dbus_error_is_set (&error)) {
+			dbus_error_free (&error);
+		}
 		goto out;
 	}
 	for (i = 0; i < num_hal_udis; i++) {
@@ -493,7 +494,9 @@ handle_mount (LibHalContext *hal_ctx,
 		dbus_error_init (&error);
 		if (libhal_device_get_property_bool (hal_ctx, udi, "volume.ignore", &error) || 
 		    dbus_error_is_set (&error)) {
-			LIBHAL_FREE_DBUS_ERROR (&error);
+			if (dbus_error_is_set (&error)) {
+				dbus_error_free (&error);
+			}
 			permission_denied_volume_ignore (device);
 		}
 
@@ -506,6 +509,8 @@ handle_mount (LibHalContext *hal_ctx,
 
 	bailout_if_in_fstab (hal_ctx, device, label, uuid);
 
+        bailout_if_drive_is_locked (hal_ctx, drive, invoked_by_syscon_name);
+
 	/* TODO: sanity check that what hal exports is correct (cf. Martin Pitt's email) */
 
 	/* read from stdin */
@@ -601,7 +606,7 @@ handle_mount (LibHalContext *hal_ctx,
 				
 			} else {
 				/* fallback - use "disk" */
-				g_snprintf (mount_point, sizeof (mount_point), "disk");
+				g_snprintf (mount_point, sizeof (mount_point), "%s", "disk");
 			}
 			
 			/* sanitize computed mount point name, e.g. replace invalid chars with '-' */
@@ -753,22 +758,24 @@ handle_mount (LibHalContext *hal_ctx,
 #endif
 
 #ifdef HAVE_POLKIT
-	if (libpolkit_is_uid_allowed_for_privilege (pol_ctx, 
-						    invoked_by_syscon_name,
-						    invoked_by_uid,
-						    privilege,
-						    udi,
-						    &allowed_by_privilege,
-						    &is_temporary_privilege,
-						    NULL) != LIBPOLKIT_RESULT_OK) {
-		printf ("cannot lookup privilege\n");
-		unknown_error ("Cannot lookup privilege from PolicyKit");
-	}
+        if (invoked_by_syscon_name != NULL) {
+                if (libpolkit_is_uid_allowed_for_privilege (pol_ctx, 
+                                                            invoked_by_syscon_name,
+                                                            invoked_by_uid,
+                                                            privilege,
+                                                            udi,
+                                                            &allowed_by_privilege,
+                                                            &is_temporary_privilege,
+                                                            NULL) != LIBPOLKIT_RESULT_OK) {
+                        printf ("cannot lookup privilege\n");
+                        unknown_error ("Cannot lookup privilege from PolicyKit");
+                }
 
-	if (!allowed_by_privilege) {
-		printf ("caller don't possess privilege\n");
-		permission_denied_privilege (privilege, invoked_by_uid);
-	}
+                if (!allowed_by_privilege) {
+                        printf ("caller don't possess privilege\n");
+                        permission_denied_privilege (privilege, invoked_by_uid);
+                }
+        }
 #endif
 
 #ifdef DEBUG
@@ -794,7 +801,7 @@ handle_mount (LibHalContext *hal_ctx,
 			printf ("Cannot chown '%s' to uid: %d, gid: %d\n", mount_dir,
 				calling_uid, calling_gid);
 			g_rmdir (mount_dir);
-			unknown_error ();
+			unknown_error ("Failed to chown mount directory");
 		}
 #endif
 	}
@@ -817,7 +824,11 @@ handle_mount (LibHalContext *hal_ctx,
 	args[na++] = mount_do_fstype;
 
 	args[na++] = "-o";
+#ifdef HAVE_UMOUNT_HAL
+	mount_option_str = g_string_new (MOUNT_OPTIONS ",uhelper=hal");
+#else
 	mount_option_str = g_string_new (MOUNT_OPTIONS);
+#endif
 	for (i = 0; given_options[i] != NULL; i++) {
 		g_string_append (mount_option_str, ",");
 		g_string_append (mount_option_str, given_options[i]);
@@ -832,7 +843,6 @@ handle_mount (LibHalContext *hal_ctx,
 	 *                      but it doesn't really matter much at this point */
 	if (!is_remount) {
 		FILE *hal_mtab;
-		char *mount_dir_escaped;
 		FILE *hal_mtab_orig;
 		int hal_mtab_orig_len;
 		int num_read;
@@ -876,7 +886,6 @@ handle_mount (LibHalContext *hal_ctx,
 			hal_mtab_buf = g_strdup ("");
 		}
 		
-		mount_dir_escaped = g_strescape (mount_dir, NULL);
 #ifdef DEBUG
 		printf ("%d: XYA creating /media/.hal-mtab~\n", getpid ());
 #endif
@@ -888,7 +897,7 @@ handle_mount (LibHalContext *hal_ctx,
 		hal_mtab_buf = g_strdup_printf ("%s%s\t%s\t0\t%s\t%s\t%s\n", 
 						hal_mtab_buf_old,
 						device, invoked_by_uid, mount_do_fstype, 
-						mount_option_commasep, mount_dir_escaped);
+						mount_option_commasep, mount_dir);
 		g_free (hal_mtab_buf_old);
 		if (hal_mtab_buf_old == NULL) {
 			unknown_error ("Out of memory appending to /media/.hal-mtab~");
@@ -898,7 +907,6 @@ handle_mount (LibHalContext *hal_ctx,
 		}
 		fclose (hal_mtab);
 		g_free (hal_mtab_buf);
-		g_free (mount_dir_escaped);
 #ifdef DEBUG
 		printf ("%d: XYA closing /media/.hal-mtab~\n", getpid ());
 #endif
@@ -1011,7 +1019,9 @@ main (int argc, char *argv[])
 	dbus_error_init (&error);
 	if ((hal_ctx = libhal_ctx_init_direct (&error)) == NULL) {
 		printf ("Cannot connect to hald\n");
-		LIBHAL_FREE_DBUS_ERROR (&error);
+		if (dbus_error_is_set (&error)) {
+			dbus_error_free (&error);
+		}
 		usage ();
 	}
 
@@ -1019,7 +1029,9 @@ main (int argc, char *argv[])
 	system_bus = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
 	if (system_bus == NULL) {
 		printf ("Cannot connect to the system bus\n");
-		LIBHAL_FREE_DBUS_ERROR (&error);
+		if (dbus_error_is_set (&error)) {
+			dbus_error_free (&error);
+		}
 		usage ();
 	}
 #ifdef HAVE_POLKIT
